@@ -314,6 +314,7 @@ default_options.register_option('tcp_timeout', 30, validator=is_integer)
 default_options.register_option('verbose', False, validator=is_bool)
 default_options.register_option('kv_store', ':inproc:', validator=is_string)
 default_options.register_option('check_interval', 20, validator=is_integer)
+default_options.register_option('serialize_method', 'json')
 
 # dataframe-related options
 default_options.register_option('dataframe.mode.use_inf_as_na', False, validator=is_bool)
@@ -380,7 +381,7 @@ default_options.register_option('worker.copy_block_size', 64 * 1024, validator=i
 default_options.register_option('worker.cuda_thread_num', 2, validator=is_integer)
 default_options.register_option('worker.transfer_block_size', 1 * 1024 ** 2, validator=is_integer)
 default_options.register_option('worker.transfer_compression', 'lz4', validator=is_string, serialize=True)
-default_options.register_option('worker.prepare_data_timeout', 600, validator=is_integer)
+default_options.register_option('worker.prepare_data_timeout', 1000, validator=is_integer)
 default_options.register_option('worker.peer_blacklist_time', 3600, validator=is_numeric, serialize=True)
 default_options.register_option('worker.io_parallel_num', 1, validator=is_integer, serialize=True)
 default_options.register_option('worker.recover_dead_process', True, validator=is_bool, serialize=True)
@@ -414,7 +415,8 @@ default_options.register_option('client.serial_type', 'arrow', validator=is_stri
 default_options.register_option('custom_log_dir', None, validator=any_validator(is_null, is_string))
 
 # vineyard
-default_options.register_option('vineyard.socket', None)
+default_options.register_option('vineyard.socket', os.environ.get('VINEYARD_IPC_SOCKET', None))
+default_options.register_option('vineyard.enabled', os.environ.get('WITH_VINEYARD', None) is not None)
 
 _options_local = threading.local()
 _options_local.default_options = default_options
