@@ -402,11 +402,14 @@ class _ExecutableMixin:
             raise ValueError(
                 f'Tileable object {key} must be executed first before {action}')
 
-    def fetch(self, session=None, **kw):
+    def _fetch(self, session=None, **kw):
         session = self._get_session(session)
         self._check_session(session, 'fetch')
         return session.fetch(self, **kw)
 
+    def fetch(self, session=None, **kw):
+        return self._fetch(session=session, **kw)
+
     def fetch_log(self, session=None, offsets=None, sizes=None):
         session = self._get_session(session)
         self._check_session(session, 'fetch_log')
@@ -423,21 +426,21 @@ class _ExecuteAndFetchMixin:
     def _execute_and_fetch(self, session=None, **kw):
         if session is None and len(self._executed_sessions) > 0:
             session = self._executed_sessions[-1]
-        try:
-            # fetch first, to reduce the potential cost of submitting a graph
-            return self.fetch(session=session)
-        except ValueError:
-            # not execute before
-            wait = kw.pop('wait', True)
 
-            def run():
-                return self.execute(session=session, **kw).fetch(session=session)
+        wait = kw.pop('wait', True)
 
-            if wait:
-                return run()
-            else:
-                thread_executor = ThreadPoolExecutor(1)
-                return thread_executor.submit(run)
+        def run():
+            fetch_kwargs = kw.pop('fetch_kwargs', dict())
+            if len(self._executed_sessions) == 0:
+                # not executed before
+                self.execute(session=session, **kw)
+            return self.fetch(session=session, **fetch_kwargs)
+
+        if wait:
+            return run()
+        else:
+            thread_executor = ThreadPoolExecutor(1)
+            return thread_executor.submit(run)
 
 
 class _ToObjectMixin(_ExecuteAndFetchMixin):
@@ -723,10 +726,38 @@ class ChunksIndexer(object):
 
 
 class ExecutableTuple(tuple, _ExecutableMixin, _ToObjectMixin):
-    def __init__(self, *_):
+    def __init__(self, *args):
         super().__init__()
+
+        self._fields_to_idx = None
+        self._fields = None
+        self._raw_type = None
+
+        if len(args) == 1 and isinstance(args[0], tuple):
+            self._fields = getattr(args[0], '_fields', None)
+            if self._fields is not None:
+                self._raw_type = type(args[0])
+                self._fields_to_idx = {f: idx for idx, f in enumerate(self._fields)}
+
         self._executed_sessions = []
 
+    def __getattr__(self, item):
+        if self._fields_to_idx is None or item not in self._fields_to_idx:
+            raise AttributeError(item)
+        return self[self._fields_to_idx[item]]
+
+    def __dir__(self):
+        result = list(super().__dir__()) + list(self._fields or [])
+        return sorted(result)
+
+    def __repr__(self):
+        if not self._fields:
+            return super().__repr__()
+        items = []
+        for k, v in zip(self._fields, self):
+            items.append(f'{k}={v!r}')
+        return '%s(%s)' % (self._raw_type.__name__, ', '.join(items))
+
     def execute(self, session=None, **kw):
         if len(self) == 0:
             return self
@@ -735,7 +766,10 @@ class ExecutableTuple(tuple, _ExecutableMixin, _ToObjectMixin):
     def fetch(self, session=None, **kw):
         if len(self) == 0:
             return tuple()
-        return super().fetch(session=session, **kw)
+        ret = super().fetch(session=session, **kw)
+        if self._raw_type is not None:
+            ret = self._raw_type(*ret)
+        return ret
 
     def fetch_log(self, session=None, offsets=None, sizes=None):
         if len(self) == 0:
