@@ -17,9 +17,11 @@
 from .initializer import DataFrame, Series, Index, named_dataframe, named_series
 # do imports to register operands
 from .base.cut import cut
+from .base.eval import mars_eval as eval  # pylint: disable=redefined-builtin
 from .base.melt import melt
 from .base.qcut import qcut
 from .datasource.from_tensor import dataframe_from_tensor, series_from_tensor
+from .datasource.from_index import series_from_index
 from .datasource.from_records import from_records
 from .datasource.from_vineyard import from_vineyard
 from .datasource.read_csv import read_csv
