@@ -18,14 +18,15 @@ import copy
 import numpy as np
 import pandas as pd
 
-from ...tensor.datasource import tensor as astensor
+from ...core import Base, Entity
 from ...serialize import AnyField, Float64Field
 from ...tensor.core import TENSOR_TYPE, ChunkData, Chunk
+from ...tensor.datasource import tensor as astensor
 from ...utils import classproperty
 from ..align import align_series_series, align_dataframe_series, align_dataframe_dataframe
 from ..core import DATAFRAME_TYPE, SERIES_TYPE, DATAFRAME_CHUNK_TYPE, SERIES_CHUNK_TYPE
-from ..operands import DataFrameOperandMixin, DataFrameOperand
 from ..initializer import Series, DataFrame
+from ..operands import DataFrameOperandMixin, DataFrameOperand
 from ..ufunc.tensor import TensorUfuncMixin
 from ..utils import parse_index, infer_dtypes, infer_dtype, infer_index_value, build_empty_df
 
@@ -206,7 +207,7 @@ class DataFrameBinOpMixin(DataFrameOperandMixin):
         new_op = op.copy()
         if isinstance(other, SERIES_TYPE):
             return new_op.new_seriess(op.inputs, other.shape, nsplits=other.nsplits, dtype=out.dtype,
-                                      index_value=other.index_value, chunks=out_chunks)
+                                      name=other.name, index_value=other.index_value, chunks=out_chunks)
         else:
             return new_op.new_dataframes(op.inputs, other.shape, nsplits=other.nsplits, dtypes=out.dtypes,
                                          index_value=other.index_value, columns_value=other.columns_value,
@@ -229,35 +230,47 @@ class DataFrameBinOpMixin(DataFrameOperandMixin):
 
     @classmethod
     def execute(cls, ctx, op):
-        if len(op.inputs) == 2:
-            df, other = ctx[op.inputs[0].key], ctx[op.inputs[1].key]
-            if isinstance(op.inputs[0], SERIES_CHUNK_TYPE) and \
-                    isinstance(op.inputs[1], DATAFRAME_CHUNK_TYPE):
-                df, other = other, df
+        if getattr(cls, '_func_name', None) is not None:
+            if len(op.inputs) == 2:
+                df, other = ctx[op.inputs[0].key], ctx[op.inputs[1].key]
+                if isinstance(op.inputs[0], SERIES_CHUNK_TYPE) and \
+                        isinstance(op.inputs[1], DATAFRAME_CHUNK_TYPE):
+                    df, other = other, df
+                    func_name = getattr(cls, '_rfunc_name')
+                else:
+                    func_name = getattr(cls, '_func_name')
+            elif pd.api.types.is_scalar(op.lhs) or isinstance(op.lhs, np.ndarray):
+                df = ctx[op.rhs.key]
+                other = op.lhs
                 func_name = getattr(cls, '_rfunc_name')
             else:
+                df = ctx[op.lhs.key]
+                other = op.rhs
                 func_name = getattr(cls, '_func_name')
-        elif pd.api.types.is_scalar(op.lhs) or isinstance(op.lhs, np.ndarray):
-            df = ctx[op.rhs.key]
-            other = op.lhs
-            func_name = getattr(cls, '_rfunc_name')
+            if df.ndim == 2:
+                kw = dict(axis=op.axis)
+            else:
+                kw = dict()
+            if op.fill_value is not None:
+                # comparison function like eq does not have `fill_value`
+                kw['fill_value'] = op.fill_value
+            if op.level is not None:
+                # logical function like and may don't have `level` (for Series type)
+                kw['level'] = op.level
+            if hasattr(other, 'ndim') and other.ndim == 0:
+                other = other.item()
+            ctx[op.outputs[0].key] = getattr(df, func_name)(other, **kw)
         else:
-            df = ctx[op.lhs.key]
-            other = op.rhs
-            func_name = getattr(cls, '_func_name')
-        if df.ndim == 2:
-            kw = dict({'axis': op.axis})
-        else:
-            kw = dict()
-        if op.fill_value is not None:
-            # comparison function like eq does not have `fill_value`
-            kw['fill_value'] = op.fill_value
-        if op.level is not None:
-            # logical function like and may don't have `level` (for Series type)
-            kw['level'] = op.level
-        if hasattr(other, 'ndim') and other.ndim == 0:
-            other = other.item()
-        ctx[op.outputs[0].key] = getattr(df, func_name)(other, **kw)
+            inputs_iter = iter(op.inputs)
+            if not pd.api.types.is_scalar(op.lhs):
+                lhs = ctx[next(inputs_iter).key]
+            else:
+                lhs = op.lhs
+            if not pd.api.types.is_scalar(op.rhs):
+                rhs = ctx[next(inputs_iter).key]
+            else:
+                rhs = op.rhs
+            ctx[op.outputs[0].key] = cls._operator(lhs, rhs)  # pylint: disable=too-many-function-args
 
     @classproperty
     def _operator(self):
@@ -285,7 +298,8 @@ class DataFrameBinOpMixin(DataFrameOperandMixin):
             x2_dtype = x2.dtype if hasattr(x2, 'dtype') else type(x2)
             dtype = infer_dtype(x1.dtype, np.dtype(x2_dtype), cls._operator)
             ret = {'shape': x1.shape, 'dtype': dtype, 'index_value': x1.index_value}
-            if pd.api.types.is_scalar(x2) or (hasattr(x2, 'ndim') and x2.ndim == 0):
+            if pd.api.types.is_scalar(x2) or (hasattr(x2, 'ndim') and (x2.ndim == 0 or
+                                                                       x2.ndim == 1)):
                 ret['name'] = x1.name
             return ret
 
@@ -520,7 +534,7 @@ class DataFrameBinOp(DataFrameOperand, DataFrameBinOpMixin):
             self._lhs = self._inputs[0]
             self._rhs = self._inputs[1]
         else:
-            if isinstance(self._lhs, (DATAFRAME_TYPE, SERIES_TYPE)):
+            if isinstance(self._lhs, (Base, Entity)):
                 self._lhs = self._inputs[0]
             elif pd.api.types.is_scalar(self._lhs):
                 self._rhs = self._inputs[0]
@@ -578,17 +592,24 @@ class DataFrameUnaryOp(DataFrameOperand, DataFrameUnaryOpMixin):
     def __init__(self, output_types=None, **kw):
         super().__init__(_output_types=output_types, **kw)
 
+    @classmethod
+    def _get_output_dtype(cls, df):
+        if df.ndim == 2:
+            return df.dtypes
+        else:
+            return df.dtype
+
     def __call__(self, df):
         self.output_types = df.op.output_types
         if df.ndim == 2:
-            return self.new_dataframe([df], shape=df.shape, dtypes=df.dtypes,
+            return self.new_dataframe([df], shape=df.shape, dtypes=self._get_output_dtype(df),
                                       columns_value=df.columns_value,
                                       index_value=df.index_value)
         else:
             series = df
             return self.new_series([series], shape=series.shape, name=series.name,
                                    index_value=series.index_value,
-                                   dtype=series.dtype)
+                                   dtype=self._get_output_dtype(series))
 
 
 class DataFrameUnaryUfunc(DataFrameUnaryOp, TensorUfuncMixin):
