@@ -14,11 +14,11 @@
 
 from ... import opcodes as OperandDef
 from ...utils import classproperty
-from .core import DataFrameBinOpMixin, DataFrameBinOp
+from .core import DataFrameBinopUfunc
 from .docstring import bin_compare_doc
 
 
-class DataFrameGreaterEqual(DataFrameBinOp, DataFrameBinOpMixin):
+class DataFrameGreaterEqual(DataFrameBinopUfunc):
     _op_type_ = OperandDef.GE
 
     _func_name = 'ge'
@@ -28,6 +28,11 @@ class DataFrameGreaterEqual(DataFrameBinOp, DataFrameBinOpMixin):
     def _operator(self):
         return lambda lhs, rhs: lhs.ge(rhs)
 
+    @classproperty
+    def tensor_op_type(self):
+        from ...tensor.arithmetic import TensorGreaterEqual
+        return TensorGreaterEqual
+
 
 _ge_example = """
 >>> a.ge(b, fill_value=0).execute()
