@@ -12,12 +12,24 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+import numpy as np
+import pandas as pd
+
 from ... import opcodes as OperandDef
 from ...utils import classproperty
 from .core import DataFrameUnaryUfunc
 
 
-class DataFrameIsNan(DataFrameUnaryUfunc):
+class DataFrameIsUFuncMixin:
+    @classmethod
+    def _get_output_dtype(cls, df):
+        if df.ndim == 2:
+            return pd.Series(np.dtype(bool), index=df.dtypes.index)
+        else:
+            return np.dtype(bool)
+
+
+class DataFrameIsNan(DataFrameIsUFuncMixin, DataFrameUnaryUfunc):
     _op_type_ = OperandDef.ISNAN
     _func_name = 'isnan'
 
@@ -27,7 +39,7 @@ class DataFrameIsNan(DataFrameUnaryUfunc):
         return TensorIsNan
 
 
-class DataFrameIsInf(DataFrameUnaryUfunc):
+class DataFrameIsInf(DataFrameIsUFuncMixin, DataFrameUnaryUfunc):
     _op_type_ = OperandDef.ISINF
     _func_name = 'isinf'
 
@@ -37,7 +49,7 @@ class DataFrameIsInf(DataFrameUnaryUfunc):
         return TensorIsInf
 
 
-class DataFrameIsFinite(DataFrameUnaryUfunc):
+class DataFrameIsFinite(DataFrameIsUFuncMixin, DataFrameUnaryUfunc):
     _op_type_ = OperandDef.ISFINITE
     _func_name = 'isfinite'
 
