@@ -16,6 +16,7 @@ import itertools
 import operator
 import re
 from copy import copy as copy_obj
+from distutils.version import LooseVersion
 from numbers import Integral
 from typing import Type, Sequence
 
@@ -31,6 +32,10 @@ from pandas.arrays import StringArray as StringArrayBase
 from pandas.core import ops
 from pandas.core.algorithms import take
 from pandas.compat import set_function_name
+try:
+    from pandas._libs.arrays import NDArrayBacked
+except ImportError:
+    NDArrayBacked = None
 try:
     import pyarrow as pa
     pa_null = pa.NULL
@@ -41,6 +46,8 @@ except ImportError:  # pragma: no cover
 from ..config import options
 from ..utils import is_kernel_mode
 
+_use_bool_any_all = LooseVersion(pd.__version__) >= '1.3.0'
+
 
 class ArrowDtype(ExtensionDtype):
     @property
@@ -105,6 +112,14 @@ class ArrowStringDtypeAlias(ArrowStringDtype):
     name = 'arrow_string'  # register an alias name for compatibility
 
 
+class ArrowListDtypeType(type):
+    """
+    the type of ArrowListDtype, this metaclass determines subclass ability
+    """
+
+    pass
+
+
 class ArrowListDtype(ArrowDtype):
     _metadata = ("_value_type",)
 
@@ -128,11 +143,11 @@ class ArrowListDtype(ArrowDtype):
 
     @property
     def kind(self):
-        return self._value_type.kind
+        return "O"
 
     @property
     def type(self):
-        return self._value_type.type
+        return ArrowListDtypeType
 
     @property
     def name(self):
@@ -238,12 +253,21 @@ class ArrowArray(ExtensionArray):
 
         self._use_arrow = True
         self._arrow_array = arrow_array
-        self._dtype = dtype
+
+        if NDArrayBacked is not None and isinstance(self, NDArrayBacked):
+            NDArrayBacked.__init__(self, np.array([]), dtype)
+        else:
+            self._dtype = dtype
 
     def _init_by_numpy(self, values, dtype: ArrowDtype = None, copy=False):
         self._use_arrow = False
-        self._ndarray = np.array(values, copy=copy)
-        self._dtype = dtype
+
+        ndarray = np.array(values, copy=copy)
+        if NDArrayBacked is not None and isinstance(self, NDArrayBacked):
+            NDArrayBacked.__init__(self, ndarray, dtype)
+        else:
+            self._dtype = dtype
+            self._ndarray = np.array(values, copy=copy)
 
     @classmethod
     def _pandas_only(cls):
@@ -515,11 +539,19 @@ class ArrowArray(ExtensionArray):
         return type(self)(series.value_counts(dropna=dropna),
                           dtype=self._dtype)
 
-    def any(self, axis=0, out=None):
-        return self.to_numpy().any(axis=axis, out=out)
+    if _use_bool_any_all:
+        def any(self, axis=0, out=None):
+            return self.to_numpy().astype(bool).any(axis=axis, out=out)
 
-    def all(self, axis=0, out=None):
-        return self.to_numpy().all(axis=axis, out=out)
+        def all(self, axis=0, out=None):
+            return self.to_numpy().astype(bool).all(axis=axis, out=out)
+
+    else:
+        def any(self, axis=0, out=None):
+            return self.to_numpy().any(axis=axis, out=out)
+
+        def all(self, axis=0, out=None):
+            return self.to_numpy().all(axis=axis, out=out)
 
     def __mars_tokenize__(self):
         if self._use_arrow:
@@ -653,6 +685,9 @@ class ArrowStringArray(ArrowArray, StringArrayBase):
 
         return set_function_name(method, f"__{op.__name__}__", cls)
 
+    def shift(self, periods: int = 1, fill_value: object = None) -> "ArrowStringArray":
+        return ExtensionArray.shift(self, periods=periods, fill_value=fill_value)
+
     @classmethod
     def _add_arithmetic_ops(cls):
         cls.__add__ = cls._create_arithmetic_method(operator.add)
@@ -759,7 +794,7 @@ class ArrowListArray(ArrowArray):
                         raise TypeError(msg)
                 else:
                     def f(x):
-                        return pd.Series(x).astype(dtype.type).tolist()
+                        return pd.Series(x).astype(dtype.value_type.type).tolist()
 
                     try:
                         arr = pd.Series(self._ndarray)
