@@ -12,7 +12,7 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from .map import map_
+from .map import series_map, index_map
 from .to_gpu import to_gpu
 from .to_cpu import to_cpu
 from .rechunk import rechunk
@@ -25,18 +25,24 @@ from .qcut import qcut
 from .shift import shift, tshift
 from .diff import df_diff, series_diff
 from .value_counts import value_counts
-from .astype import astype
+from .astype import astype, index_astype
 from .drop import df_drop, df_pop, series_drop, index_drop
 from .drop_duplicates import df_drop_duplicates, \
     series_drop_duplicates, index_drop_duplicates
+from .duplicated import df_duplicated, series_duplicated, index_duplicated
 from .melt import melt
-from .memory_usage import df_memory_usage, series_memory_usage, index_memory_usage
+from .memory_usage import df_memory_usage, series_memory_usage, \
+    index_memory_usage
 from .select_dtypes import select_dtypes
 from .map_chunk import map_chunk
 from .cartesian_chunk import cartesian_chunk
 from .rebalance import rebalance
 from .stack import stack
 from .explode import df_explode, series_explode
+from .eval import df_eval, df_query
+from .check_monotonic import check_monotonic, is_monotonic, \
+    is_monotonic_increasing, is_monotonic_decreasing
+from .pct_change import pct_change
 
 
 def _install():
@@ -62,6 +68,7 @@ def _install():
         setattr(t, 'pop', df_pop)
         setattr(t, '__delitem__', lambda df, items: df_drop(df, items, axis=1, inplace=True))
         setattr(t, 'drop_duplicates', df_drop_duplicates)
+        setattr(t, 'duplicated', df_duplicated)
         setattr(t, 'melt', melt)
         setattr(t, 'memory_usage', df_memory_usage)
         setattr(t, 'select_dtypes', select_dtypes)
@@ -70,12 +77,15 @@ def _install():
         setattr(t, 'rebalance', rebalance)
         setattr(t, 'stack', stack)
         setattr(t, 'explode', df_explode)
+        setattr(t, 'eval', df_eval)
+        setattr(t, 'query', df_query)
+        setattr(t, 'pct_change', pct_change)
 
     for t in SERIES_TYPE:
         setattr(t, 'to_gpu', to_gpu)
         setattr(t, 'to_cpu', to_cpu)
         setattr(t, 'rechunk', rechunk)
-        setattr(t, 'map', map_)
+        setattr(t, 'map', series_map)
         setattr(t, 'describe', describe)
         setattr(t, 'apply', series_apply)
         setattr(t, 'transform', series_transform)
@@ -87,18 +97,32 @@ def _install():
         setattr(t, 'astype', astype)
         setattr(t, 'drop', series_drop)
         setattr(t, 'drop_duplicates', series_drop_duplicates)
+        setattr(t, 'duplicated', series_duplicated)
         setattr(t, 'memory_usage', series_memory_usage)
         setattr(t, 'map_chunk', map_chunk)
         setattr(t, 'cartesian_chunk', cartesian_chunk)
         setattr(t, 'rebalance', rebalance)
         setattr(t, 'explode', series_explode)
+        setattr(t, 'check_monotonic', check_monotonic)
+        setattr(t, 'is_monotonic', property(fget=is_monotonic))
+        setattr(t, 'is_monotonic_increasing', property(fget=is_monotonic_increasing))
+        setattr(t, 'is_monotonic_decreasing', property(fget=is_monotonic_decreasing))
+        setattr(t, 'pct_change', pct_change)
 
     for t in INDEX_TYPE:
+        setattr(t, 'map', index_map)
         setattr(t, 'rechunk', rechunk)
         setattr(t, 'rebalance', rebalance)
         setattr(t, 'drop', index_drop)
         setattr(t, 'drop_duplicates', index_drop_duplicates)
+        setattr(t, 'duplicated', index_duplicated)
         setattr(t, 'memory_usage', index_memory_usage)
+        setattr(t, 'astype', index_astype)
+        setattr(t, 'value_counts', value_counts)
+        setattr(t, 'check_monotonic', check_monotonic)
+        setattr(t, 'is_monotonic', property(fget=is_monotonic))
+        setattr(t, 'is_monotonic_increasing', property(fget=is_monotonic_increasing))
+        setattr(t, 'is_monotonic_decreasing', property(fget=is_monotonic_decreasing))
 
     for method in _string_method_to_handlers:
         if not hasattr(StringAccessor, method):
