@@ -23,10 +23,10 @@ from ...core import OutputType
 from ...custom_log import redirect_custom_log
 from ...serialize import StringField, AnyField, BoolField, \
     TupleField, DictField, FunctionField
-from ...utils import enter_current_session
+from ...utils import enter_current_session, quiet_stdio
 from ..operands import DataFrameOperandMixin, DataFrameOperand
 from ..utils import build_df, build_series, parse_index, validate_axis, \
-    validate_output_types, quiet_stdio
+    validate_output_types, make_dtypes, make_dtype
 
 
 class ApplyOperand(DataFrameOperand, DataFrameOperandMixin):
@@ -196,7 +196,7 @@ class ApplyOperand(DataFrameOperand, DataFrameOperandMixin):
         else:
             return cls._tile_series(op)
 
-    def _infer_df_func_returns(self, df, dtypes, index):
+    def _infer_df_func_returns(self, df, dtypes, dtype=None, name=None, index=None):
         if isinstance(self._func, np.ufunc):
             output_type, new_dtypes, index_value, new_elementwise = \
                 OutputType.dataframe, None, 'inherit', True
@@ -219,7 +219,7 @@ class ApplyOperand(DataFrameOperand, DataFrameOperandMixin):
                 new_dtypes = new_dtypes or infer_df.dtypes
             else:
                 output_type = output_type or OutputType.series
-                new_dtypes = new_dtypes or infer_df.dtype
+                new_dtypes = (name or infer_df.name, dtype or infer_df.dtype)
             new_elementwise = False if new_elementwise is None else new_elementwise
         except:  # noqa: E722  # nosec
             pass
@@ -230,8 +230,11 @@ class ApplyOperand(DataFrameOperand, DataFrameOperandMixin):
         self._elementwise = new_elementwise if self._elementwise is None else self._elementwise
         return dtypes, index_value
 
-    def _call_dataframe(self, df, dtypes=None, index=None):
-        dtypes, index_value = self._infer_df_func_returns(df, dtypes, index)
+    def _call_dataframe(self, df, dtypes=None, dtype=None, name=None, index=None):
+        # for backward compatibility
+        dtype = dtype if dtype is not None else dtypes
+        dtypes, index_value = self._infer_df_func_returns(
+            df, dtypes, dtype=dtype, name=name, index=index)
         if index_value is None:
             index_value = parse_index(None, (df.key, df.index_value.key))
         for arg, desc in zip((self.output_types, dtypes), ('output_types', 'dtypes')):
@@ -259,9 +262,12 @@ class ApplyOperand(DataFrameOperand, DataFrameOperandMixin):
                 return self.new_dataframe([df], shape=shape, dtypes=dtypes, index_value=df.index_value,
                                           columns_value=parse_index(dtypes.index))
         else:
-            return self.new_series([df], shape=shape, dtype=dtypes, index_value=index_value)
+            name, dtype = dtypes
+            return self.new_series([df], shape=shape, name=name, dtype=dtype, index_value=index_value)
 
-    def _call_series(self, series, dtype=None, index=None):
+    def _call_series(self, series, dtypes=None, dtype=None, name=None, index=None):
+        # for backward compatibility
+        dtype = dtype if dtype is not None else dtypes
         if self._convert_dtype:
             try:
                 test_series = build_series(series, size=2, name=series.name)
@@ -280,13 +286,12 @@ class ApplyOperand(DataFrameOperand, DataFrameOperandMixin):
                 index_value = parse_index(None, series)
 
             if output_type == OutputType.dataframe:
-                if dtype is not None:
-                    dtypes = dtype
-                elif infer_series is not None and infer_series.ndim == 2:
-                    dtypes = infer_series.dtypes
-                else:
-                    raise TypeError('Cannot determine dtypes, '
-                                    'please specify `dtypes` as argument')
+                if dtypes is None:
+                    if infer_series is not None and infer_series.ndim == 2:
+                        dtypes = infer_series.dtypes
+                    else:
+                        raise TypeError('Cannot determine dtypes, '
+                                        'please specify `dtypes` as argument')
 
                 columns_value = parse_index(dtypes.index, store_data=True)
 
@@ -297,30 +302,197 @@ class ApplyOperand(DataFrameOperand, DataFrameOperandMixin):
                 if dtype is None and infer_series is not None and infer_series.ndim == 1:
                     dtype = infer_series.dtype
                 else:
-                    dtype = np.dtype(object)
+                    dtype = dtype if dtype is not None else np.dtype(object)
                 if infer_series is not None and infer_series.ndim == 1:
-                    name = infer_series.name
-                else:
-                    name = None
+                    name = name or infer_series.name
                 return self.new_series([series], dtype=dtype, shape=series.shape,
                                        index_value=index_value, name=name)
         else:
-            dtype, name = np.dtype('object'), None
+            dtype = dtype if dtype is not None else np.dtype('object')
             return self.new_series([series], dtype=dtype, shape=series.shape,
                                    index_value=series.index_value, name=name)
 
-    def __call__(self, df_or_series, dtypes=None, index=None):
+    def __call__(self, df_or_series, dtypes=None, dtype=None, name=None, index=None):
         axis = getattr(self, 'axis', None) or 0
+        dtypes = make_dtypes(dtypes)
+        dtype = make_dtype(dtype)
         self._axis = validate_axis(axis, df_or_series)
 
         if df_or_series.op.output_types[0] == OutputType.dataframe:
             return self._call_dataframe(df_or_series, dtypes=dtypes, index=index)
         else:
-            return self._call_series(df_or_series, dtype=dtypes, index=index)
+            return self._call_series(df_or_series, dtypes=dtypes, dtype=dtype, name=name, index=index)
 
 
 def df_apply(df, func, axis=0, raw=False, result_type=None, args=(), dtypes=None,
-             output_type=None, index=None, elementwise=None, **kwds):
+             dtype=None, name=None, output_type=None, index=None, elementwise=None,
+             **kwds):
+    """
+    Apply a function along an axis of the DataFrame.
+
+    Objects passed to the function are Series objects whose index is
+    either the DataFrame's index (``axis=0``) or the DataFrame's columns
+    (``axis=1``). By default (``result_type=None``), the final return type
+    is inferred from the return type of the applied function. Otherwise,
+    it depends on the `result_type` argument.
+
+    Parameters
+    ----------
+    func : function
+        Function to apply to each column or row.
+    axis : {0 or 'index', 1 or 'columns'}, default 0
+        Axis along which the function is applied:
+
+        * 0 or 'index': apply function to each column.
+        * 1 or 'columns': apply function to each row.
+
+    raw : bool, default False
+        Determines if row or column is passed as a Series or ndarray object:
+
+        * ``False`` : passes each row or column as a Series to the
+          function.
+        * ``True`` : the passed function will receive ndarray objects
+          instead.
+          If you are just applying a NumPy reduction function this will
+          achieve much better performance.
+
+    result_type : {'expand', 'reduce', 'broadcast', None}, default None
+        These only act when ``axis=1`` (columns):
+
+        * 'expand' : list-like results will be turned into columns.
+        * 'reduce' : returns a Series if possible rather than expanding
+          list-like results. This is the opposite of 'expand'.
+        * 'broadcast' : results will be broadcast to the original shape
+          of the DataFrame, the original index and columns will be
+          retained.
+
+        The default behaviour (None) depends on the return value of the
+        applied function: list-like results will be returned as a Series
+        of those. However if the apply function returns a Series these
+        are expanded to columns.
+
+    output_type : {'dataframe', 'series'}, default None
+        Specify type of returned object. See `Notes` for more details.
+
+    dtypes : Series, default None
+        Specify dtypes of returned DataFrames. See `Notes` for more details.
+
+    dtype : numpy.dtype, default None
+        Specify dtype of returned Series. See `Notes` for more details.
+
+    name : str, default None
+        Specify name of returned Series. See `Notes` for more details.
+
+    index : Index, default None
+        Specify index of returned object. See `Notes` for more details.
+
+    elementwise : bool, default False
+        Specify whether ``func`` is an elementwise function:
+
+        * ``False`` : The function is not elementwise. Mars will try
+          concatenating chunks in rows (when ``axis=0``) or in columns
+          (when ``axis=1``) and then apply ``func`` onto the concatenated
+          chunk. The concatenation step can cause extra latency.
+        * ``True`` : The function is elementwise. Mars will apply
+          ``func`` to original chunks. This will not introduce extra
+          concatenation step and reduce overhead.
+
+    args : tuple
+        Positional arguments to pass to `func` in addition to the
+        array/series.
+
+    **kwds
+        Additional keyword arguments to pass as keywords arguments to
+        `func`.
+
+    Returns
+    -------
+    Series or DataFrame
+        Result of applying ``func`` along the given axis of the
+        DataFrame.
+
+    See Also
+    --------
+    DataFrame.applymap: For elementwise operations.
+    DataFrame.aggregate: Only perform aggregating type operations.
+    DataFrame.transform: Only perform transforming type operations.
+
+    Notes
+    -----
+    When deciding output dtypes and shape of the return value, Mars will
+    try applying ``func`` onto a mock DataFrame,  and the apply call may
+    fail. When this happens, you need to specify the type of apply call
+    (DataFrame or Series) in output_type.
+
+    * For DataFrame output, you need to specify a list or a pandas Series
+      as ``dtypes`` of output DataFrame. ``index`` of output can also be
+      specified.
+    * For Series output, you need to specify ``dtype`` and ``name`` of
+      output Series.
+
+    Examples
+    --------
+    >>> import numpy as np
+    >>> import mars.tensor as mt
+    >>> import mars.dataframe as md
+    >>> df = md.DataFrame([[4, 9]] * 3, columns=['A', 'B'])
+    >>> df.execute()
+       A  B
+    0  4  9
+    1  4  9
+    2  4  9
+
+    Using a reducing function on either axis
+
+    >>> df.apply(np.sum, axis=0).execute()
+    A    12
+    B    27
+    dtype: int64
+
+    >>> df.apply(np.sum, axis=1).execute()
+    0    13
+    1    13
+    2    13
+    dtype: int64
+
+    Returning a list-like will result in a Series
+
+    >>> df.apply(lambda x: [1, 2], axis=1).execute()
+    0    [1, 2]
+    1    [1, 2]
+    2    [1, 2]
+    dtype: object
+
+    Passing ``result_type='expand'`` will expand list-like results
+    to columns of a Dataframe
+
+    >>> df.apply(lambda x: [1, 2], axis=1, result_type='expand').execute()
+       0  1
+    0  1  2
+    1  1  2
+    2  1  2
+
+    Returning a Series inside the function is similar to passing
+    ``result_type='expand'``. The resulting column names
+    will be the Series index.
+
+    >>> df.apply(lambda x: md.Series([1, 2], index=['foo', 'bar']), axis=1).execute()
+       foo  bar
+    0    1    2
+    1    1    2
+    2    1    2
+
+    Passing ``result_type='broadcast'`` will ensure the same shape
+    result, whether list-like or scalar is returned by the function,
+    and broadcast it along the axis. The resulting column names will
+    be the originals.
+
+    >>> df.apply(lambda x: [1, 2], axis=1, result_type='broadcast').execute()
+       A  B
+    0  1  2
+    1  1  2
+    2  1  2
+    """
     if isinstance(func, (list, dict)):
         return df.aggregate(func)
 
@@ -340,11 +512,132 @@ def df_apply(df, func, axis=0, raw=False, result_type=None, args=(), dtypes=None
 
     op = ApplyOperand(func=func, axis=axis, raw=raw, result_type=result_type, args=args, kwds=kwds,
                       output_type=output_type, elementwise=elementwise)
-    return op(df, dtypes=dtypes, index=index)
+    return op(df, dtypes=dtypes, dtype=dtype, name=name, index=index)
 
 
-def series_apply(series, func, convert_dtype=True, output_type=None,
-                 args=(), index=None, **kwds):
+def series_apply(series, func, convert_dtype=True, output_type=None, args=(),
+                 dtypes=None, dtype=None, name=None, index=None, **kwds):
+    """
+    Invoke function on values of Series.
+
+    Can be ufunc (a NumPy function that applies to the entire Series)
+    or a Python function that only works on single values.
+
+    Parameters
+    ----------
+    func : function
+        Python function or NumPy ufunc to apply.
+
+    convert_dtype : bool, default True
+        Try to find better dtype for elementwise function results. If
+        False, leave as dtype=object.
+
+    output_type : {'dataframe', 'series'}, default None
+        Specify type of returned object. See `Notes` for more details.
+
+    dtypes : Series, default None
+        Specify dtypes of returned DataFrames. See `Notes` for more details.
+
+    dtype : numpy.dtype, default None
+        Specify dtype of returned Series. See `Notes` for more details.
+
+    name : str, default None
+        Specify name of returned Series. See `Notes` for more details.
+
+    index : Index, default None
+        Specify index of returned object. See `Notes` for more details.
+
+    args : tuple
+        Positional arguments passed to func after the series value.
+
+    **kwds
+        Additional keyword arguments passed to func.
+
+    Returns
+    -------
+    Series or DataFrame
+        If func returns a Series object the result will be a DataFrame.
+
+    See Also
+    --------
+    Series.map: For element-wise operations.
+    Series.agg: Only perform aggregating type operations.
+    Series.transform: Only perform transforming type operations.
+
+    Notes
+    -----
+    When deciding output dtypes and shape of the return value, Mars will
+    try applying ``func`` onto a mock Series, and the apply call may fail.
+    When this happens, you need to specify the type of apply call
+    (DataFrame or Series) in output_type.
+
+    * For DataFrame output, you need to specify a list or a pandas Series
+      as ``dtypes`` of output DataFrame. ``index`` of output can also be
+      specified.
+    * For Series output, you need to specify ``dtype`` and ``name`` of
+      output Series.
+
+    Examples
+    --------
+    Create a series with typical summer temperatures for each city.
+
+    >>> import mars.tensor as mt
+    >>> import mars.dataframe as md
+    >>> s = md.Series([20, 21, 12],
+    ...               index=['London', 'New York', 'Helsinki'])
+    >>> s.execute()
+    London      20
+    New York    21
+    Helsinki    12
+    dtype: int64
+
+    Square the values by defining a function and passing it as an
+    argument to ``apply()``.
+
+    >>> def square(x):
+    ...     return x ** 2
+    >>> s.apply(square).execute()
+    London      400
+    New York    441
+    Helsinki    144
+    dtype: int64
+
+    Square the values by passing an anonymous function as an
+    argument to ``apply()``.
+
+    >>> s.apply(lambda x: x ** 2).execute()
+    London      400
+    New York    441
+    Helsinki    144
+    dtype: int64
+
+    Define a custom function that needs additional positional
+    arguments and pass these additional arguments using the
+    ``args`` keyword.
+
+    >>> def subtract_custom_value(x, custom_value):
+    ...     return x - custom_value
+
+    >>> s.apply(subtract_custom_value, args=(5,)).execute()
+    London      15
+    New York    16
+    Helsinki     7
+    dtype: int64
+
+    Define a custom function that takes keyword arguments
+    and pass these arguments to ``apply``.
+
+    >>> def add_custom_values(x, **kwargs):
+    ...     for month in kwargs:
+    ...         x += kwargs[month]
+    ...     return x
+
+    >>> s.apply(add_custom_values, june=30, july=20, august=25).execute()
+    London      95
+    New York    96
+    Helsinki    87
+    dtype: int64
+    """
     if isinstance(func, (list, dict)):
         return series.aggregate(func)
 
@@ -364,8 +657,7 @@ def series_apply(series, func, convert_dtype=True, output_type=None,
     output_types = validate_output_types(
         output_type=output_type, output_types=output_types, object_type=object_type)
     output_type = output_types[0] if output_types else OutputType.series
-    dtypes = kwds.pop('dtypes', kwds.pop('dtype', None))
 
     op = ApplyOperand(func=func, convert_dtype=convert_dtype, args=args, kwds=kwds,
                       output_type=output_type)
-    return op(series, dtypes=dtypes, index=index)
+    return op(series, dtypes=dtypes, dtype=dtype, name=name, index=index)
