@@ -12,6 +12,8 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+from distutils.version import LooseVersion
+
 import numpy as np
 import pandas as pd
 
@@ -20,11 +22,11 @@ from ...config import options
 from ...core import OutputType
 from ...custom_log import redirect_custom_log
 from ...serialize import AnyField, BoolField, TupleField, DictField, StringField
-from ...utils import enter_current_session
+from ...utils import enter_current_session, quiet_stdio
 from ..core import DATAFRAME_CHUNK_TYPE, DATAFRAME_TYPE
 from ..operands import DataFrameOperandMixin, DataFrameOperand
 from ..utils import build_df, build_series, validate_axis, \
-    parse_index, filter_dtypes_by_index, make_dtypes, quiet_stdio
+    parse_index, filter_dtypes_by_index, make_dtypes
 
 
 class TransformOperand(DataFrameOperand, DataFrameOperandMixin):
@@ -141,6 +143,8 @@ class TransformOperand(DataFrameOperand, DataFrameOperandMixin):
 
                     if op.call_agg:
                         new_shape[op.axis] = np.nan
+                        params['index_value'] = parse_index(
+                            None, c.key, c.index_value.key)
                     new_columns_value = parse_index(new_dtypes.index)
                 else:
                     new_dtypes = out_df.dtypes
@@ -197,8 +201,11 @@ class TransformOperand(DataFrameOperand, DataFrameOperandMixin):
                     if self.call_agg:
                         infer_df = test_df.agg(self._func, args=self.args, **self.kwds)
                     else:
-                        infer_df = test_df.transform(self._func, convert_dtype=self.convert_dtype,
-                                                     args=self.args, **self.kwds)
+                        if LooseVersion(pd.__version__) >= '1.2.0':
+                            infer_df = test_df.transform(self._func, *self.args, **self.kwds)
+                        else:  # pragma: no cover
+                            infer_df = test_df.transform(self._func, convert_dtype=self.convert_dtype,
+                                                         args=self.args, **self.kwds)
             except:  # noqa: E722
                 infer_df = None
 
@@ -363,9 +370,9 @@ def series_transform(series, func, convert_dtype=True, axis=0, *args, dtype=None
         Specify dtypes of returned DataFrames. See `Notes` for more details.
 
     *args
-    Positional arguments to pass to `func`.
+        Positional arguments to pass to `func`.
     **kwargs
-    Keyword arguments to pass to `func`.
+        Keyword arguments to pass to `func`.
 
     Returns
     -------
