@@ -14,6 +14,7 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+import weakref
 from collections.abc import Iterable
 from io import StringIO
 from typing import Union
@@ -88,6 +89,10 @@ class IndexValue(Serializable):
         def key(self):
             return self._key
 
+        @property
+        def inferred_type(self):
+            return None
+
         def to_pandas(self):
             kw = {field.tag_name(None): getattr(self, attr, None)
                   for attr, field in self._FIELDS.items()
@@ -126,11 +131,19 @@ class IndexValue(Serializable):
         _categories = ListField('categories')
         _ordered = BoolField('ordered')
 
+        @property
+        def inferred_type(self):
+            return 'categorical'
+
     class IntervalIndex(IndexBase):
         _name = AnyField('name')
         _data = IntervalArrayField('data')
         _closed = StringField('closed')
 
+        @property
+        def inferred_type(self):
+            return 'interval'
+
     class DatetimeIndex(IndexBase):
         _name = AnyField('name')
         _data = NDArrayField('data')
@@ -144,6 +157,10 @@ class IndexValue(Serializable):
         _dayfirst = BoolField('dayfirst')
         _yearfirst = BoolField('yearfirst')
 
+        @property
+        def inferred_type(self):
+            return 'datetime64'
+
         @property
         def freq(self):
             return getattr(self, '_freq', None)
@@ -158,6 +175,10 @@ class IndexValue(Serializable):
         _end = AnyField('end')
         _closed = AnyField('closed')
 
+        @property
+        def inferred_type(self):
+            return 'timedelta64'
+
     class PeriodIndex(IndexBase):
         _name = AnyField('name')
         _data = NDArrayField('data')
@@ -175,27 +196,47 @@ class IndexValue(Serializable):
         _tz = AnyField('tz')
         _dtype = DataTypeField('dtype')
 
+        @property
+        def inferred_type(self):
+            return 'period'
+
     class Int64Index(IndexBase):
         _name = AnyField('name')
         _data = NDArrayField('data')
         _dtype = DataTypeField('dtype')
 
+        @property
+        def inferred_type(self):
+            return 'integer'
+
     class UInt64Index(IndexBase):
         _name = AnyField('name')
         _data = NDArrayField('data')
         _dtype = DataTypeField('dtype')
 
+        @property
+        def inferred_type(self):
+            return 'integer'
+
     class Float64Index(IndexBase):
         _name = AnyField('name')
         _data = NDArrayField('data')
         _dtype = DataTypeField('dtype')
 
+        @property
+        def inferred_type(self):
+            return 'floating'
+
     class MultiIndex(IndexBase):
         _names = ListField('names', on_serialize=list)
         _dtypes = ListField('dtypes', on_serialize=list)
         _data = NDArrayField('data')
         _sortorder = Int32Field('sortorder')
 
+        @property
+        def inferred_type(self):
+            return 'mixed'
+
         @property
         def names(self) -> list:
             return self._names
@@ -280,6 +321,10 @@ class IndexValue(Serializable):
     def name(self):
         return getattr(self._index_value, '_name', None)
 
+    @property
+    def inferred_type(self):
+        return self._index_value.inferred_type
+
     def has_value(self):
         if isinstance(self._index_value, self.RangeIndex):
             if np.isnan(self._index_value.max_val):
@@ -378,21 +423,60 @@ class _ToPandasMixin(_ExecuteAndFetchMixin):
         return self._execute_and_fetch(session=session, **kw)
 
 
+class _BatchedFetcher:
+    __slots__ = ()
+
+    def _iter(self, batch_size=1000, session=None):
+        from .indexing.iloc import iloc
+
+        size = self.shape[0]
+        n_batch = ceildiv(size, batch_size)
+
+        if n_batch > 1:
+            for i in range(n_batch):
+                batch_data = iloc(self)[batch_size * i: batch_size * (i + 1)]
+                yield batch_data._fetch(session=session)
+        else:
+            yield self._fetch(session=session)
+
+    def iterbatch(self, batch_size=1000, session=None, **kw):
+        # trigger execution
+        self.execute(session=session, **kw)
+        return self._iter(batch_size=batch_size, session=session)
+
+    def fetch(self, session=None, **kw):
+        from .indexing.iloc import DataFrameIlocGetItem, SeriesIlocGetItem
+
+        batch_size = kw.pop('batch_size', 1000)
+        if len(kw) > 0:  # pragma: no cover
+            raise TypeError(
+                f"'{next(iter(kw))}' is an invalid keyword argument for this function")
+
+        if isinstance(self.op, (DataFrameIlocGetItem, SeriesIlocGetItem)):
+            # see GH#1871
+            # already iloc, do not trigger batch fetch
+            return self._fetch(session=session, **kw)
+        else:
+            batches = list(self._iter(batch_size=batch_size, session=session))
+            return pd.concat(batches) if len(batches) > 1 else batches[0]
+
+
 class IndexData(HasShapeTileableData, _ToPandasMixin):
     __slots__ = ()
 
     # optional field
     _dtype = DataTypeField('dtype')
     _name = AnyField('name')
+    _names = AnyField('names')
     _index_value = ReferenceField('index_value', IndexValue, on_deserialize=_on_deserialize_index_value)
     _chunks = ListField('chunks', ValueType.reference(IndexChunkData),
                         on_serialize=lambda x: [it.data for it in x] if x is not None else x,
                         on_deserialize=lambda x: [IndexChunk(it) for it in x] if x is not None else x)
 
     def __init__(self, op=None, shape=None, nsplits=None, dtype=None,
-                 name=None, index_value=None, chunks=None, **kw):
+                 name=None, names=None, index_value=None, chunks=None, **kw):
         super().__init__(_op=op, _shape=shape, _nsplits=nsplits, _dtype=dtype, _name=name,
-                         _index_value=index_value, _chunks=chunks, **kw)
+                         _names=names, _index_value=index_value, _chunks=chunks, **kw)
 
     @classmethod
     def cls(cls, provider):
@@ -428,6 +512,14 @@ class IndexData(HasShapeTileableData, _ToPandasMixin):
     def __repr__(self):
         return self._to_str(representation=True)
 
+    def _to_mars_tensor(self, dtype=None, order='K', extract_multi_index=False):
+        tensor = self.to_tensor(extract_multi_index=extract_multi_index)
+        dtype = dtype if dtype is not None else tensor.dtype
+        return tensor.astype(dtype=dtype, order=order, copy=False)
+
+    def __mars_tensor__(self, dtype=None, order='K'):
+        return self._to_mars_tensor(dtype=dtype, order=order)
+
     @property
     def dtype(self):
         return getattr(self, '_dtype', None) or self.op.dtype
@@ -436,17 +528,25 @@ class IndexData(HasShapeTileableData, _ToPandasMixin):
     def name(self):
         return self._name
 
+    @property
+    def names(self):
+        return getattr(self, '_names', None) or [self.name]
+
     @property
     def index_value(self) -> IndexValue:
         return self._index_value
 
+    @property
+    def inferred_type(self):
+        return self._index_value.inferred_type
+
     def to_tensor(self, dtype=None, extract_multi_index=False):
         from ..tensor.datasource.from_dataframe import from_index
         return from_index(self, dtype=dtype, extract_multi_index=extract_multi_index)
 
 
 class Index(HasShapeTileableEnity, _ToPandasMixin):
-    __slots__ = ()
+    __slots__ = '_df_or_series', '_parent_key', '_axis'
     _allow_data_type_ = (IndexData,)
 
     def __new__(cls, data: Union[pd.Index, IndexData], **_):
@@ -461,13 +561,49 @@ class Index(HasShapeTileableEnity, _ToPandasMixin):
     def __len__(self):
         return len(self._data)
 
-    def _to_mars_tensor(self, dtype=None, order='K', extract_multi_index=False):
-        tensor = self._data.to_tensor(extract_multi_index=extract_multi_index)
-        dtype = dtype if dtype is not None else tensor.dtype
-        return tensor.astype(dtype=dtype, order=order, copy=False)
-
     def __mars_tensor__(self, dtype=None, order='K'):
-        return self._to_mars_tensor(dtype=dtype, order=order)
+        return self._data.__mars_tensor__(dtype=dtype, order=order)
+
+    def _get_df_or_series(self):
+        obj = getattr(self, '_df_or_series', None)
+        if obj is not None:
+            return obj()
+        return None
+
+    def _set_df_or_series(self, df_or_series, axis):
+        self._df_or_series = weakref.ref(df_or_series)
+        self._parent_key = df_or_series.key
+        self._axis = axis
+
+    @property
+    def name(self):
+        return self._data.name
+
+    @name.setter
+    def name(self, value):
+        df_or_series = self._get_df_or_series()
+        if df_or_series is not None and df_or_series.key == self._parent_key:
+            df_or_series.rename_axis(value, axis=self._axis, inplace=True)
+            self.data = df_or_series.axes[self._axis].data
+        else:
+            self.rename(value, inplace=True)
+
+    @property
+    def names(self):
+        return self._data.names
+
+    @names.setter
+    def names(self, value):
+        df_or_series = self._get_df_or_series()
+        if df_or_series is not None:
+            df_or_series.rename_axis(value, axis=self._axis, inplace=True)
+            self.data = df_or_series.axes[self._axis].data
+        else:
+            self.rename(value, inplace=True)
+
+    @property
+    def values(self):
+        return self.to_tensor()
 
     def to_frame(self, index: bool = True, name=None):
         """
@@ -535,7 +671,7 @@ class Index(HasShapeTileableEnity, _ToPandasMixin):
         else:
             columns = [name or self.name or 0]
         index_ = self if index else None
-        return dataframe_from_tensor(self._to_mars_tensor(self, extract_multi_index=True),
+        return dataframe_from_tensor(self._data._to_mars_tensor(self, extract_multi_index=True),
                                      index=index_, columns=columns)
 
     def to_series(self, index=None, name=None):
@@ -557,12 +693,9 @@ class Index(HasShapeTileableEnity, _ToPandasMixin):
         Series
             The dtype will be based on the type of the Index values.
         """
-        from ..tensor import tensor as astensor
-        from . import series_from_tensor
+        from . import series_from_index
 
-        name = name or self.name or 0
-        index_ = index if index is not None else self
-        return series_from_tensor(astensor(self), index=index_, name=name)
+        return series_from_index(self, index=index, name=name)
 
 
 class RangeIndex(Index):
@@ -748,6 +881,13 @@ class BaseSeriesData(HasShapeTileableData, _ToPandasMixin):
     def axes(self):
         return [self.index]
 
+    @property
+    def empty(self):
+        shape = getattr(self, '_shape')
+        if np.any(np.isnan(shape)):
+            raise ValueError('Tileable object must be executed first')
+        return shape == (0,)
+
     def to_tensor(self, dtype=None):
         from ..tensor.datasource.from_dataframe import from_series
         return from_series(self, dtype=dtype)
@@ -758,9 +898,14 @@ class BaseSeriesData(HasShapeTileableData, _ToPandasMixin):
         return series_from_tensor(in_tensor, index=index, name=name)
 
 
-class SeriesData(BaseSeriesData):
+class SeriesData(_BatchedFetcher, BaseSeriesData):
     _type_name = 'Series'
 
+    def __mars_tensor__(self, dtype=None, order='K'):
+        tensor = self.to_tensor()
+        dtype = dtype if dtype is not None else tensor.dtype
+        return tensor.astype(dtype=dtype, order=order, copy=False)
+
     @classmethod
     def cls(cls, provider):
         if provider.type == ProviderType.protobuf:
@@ -768,6 +913,16 @@ class SeriesData(BaseSeriesData):
             return SeriesDef
         return super().cls(provider)
 
+    def iteritems(self, batch_size=10000, session=None):
+        for batch_data in self.iterbatch(batch_size=batch_size, session=session):
+            yield from getattr(batch_data, 'iteritems')()
+
+    items = iteritems
+
+    def to_dict(self, into=dict, batch_size=10000, session=None):
+        fetch_kwargs = dict(batch_size=batch_size)
+        return self.to_pandas(session=session, fetch_kwargs=fetch_kwargs).to_dict(into=into)
+
 
 class Series(HasShapeTileableEnity, _ToPandasMixin):
     __slots__ = '_cache',
@@ -808,7 +963,13 @@ class Series(HasShapeTileableEnity, _ToPandasMixin):
         """
         The index (axis labels) of the Series.
         """
-        return self._data.index
+        idx = self._data.index
+        idx._set_df_or_series(self, 0)
+        return idx
+
+    @index.setter
+    def index(self, new_index):
+        self.set_axis(new_index, axis=0, inplace=True)
 
     @property
     def name(self):
@@ -862,9 +1023,86 @@ class Series(HasShapeTileableEnity, _ToPandasMixin):
         return len(self._data)
 
     def __mars_tensor__(self, dtype=None, order='K'):
-        tensor = self._data.to_tensor()
-        dtype = dtype if dtype is not None else tensor.dtype
-        return tensor.astype(dtype=dtype, order=order, copy=False)
+        return self._data.__mars_tensor__(dtype=dtype, order=order)
+
+    def keys(self):
+        """
+        Return alias for index.
+
+        Returns
+        -------
+        Index
+            Index of the Series.
+        """
+        return self.index
+
+    @property
+    def values(self):
+        return self.to_tensor()
+
+    def iteritems(self, batch_size=10000, session=None):
+        """
+        Lazily iterate over (index, value) tuples.
+
+        This method returns an iterable tuple (index, value). This is
+        convenient if you want to create a lazy iterator.
+
+        Returns
+        -------
+        iterable
+            Iterable of tuples containing the (index, value) pairs from a
+            Series.
+
+        See Also
+        --------
+        DataFrame.items : Iterate over (column name, Series) pairs.
+        DataFrame.iterrows : Iterate over DataFrame rows as (index, Series) pairs.
+
+        Examples
+        --------
+        >>> import mars.dataframe as md
+        >>> s = md.Series(['A', 'B', 'C'])
+        >>> for index, value in s.items():
+        ...     print(f"Index : {index}, Value : {value}")
+        Index : 0, Value : A
+        Index : 1, Value : B
+        Index : 2, Value : C
+        """
+        return self._data.iteritems(batch_size=batch_size, session=session)
+
+    items = iteritems
+
+    def to_dict(self, into=dict, batch_size=10000, session=None):
+        """
+        Convert Series to {label -> value} dict or dict-like object.
+
+        Parameters
+        ----------
+        into : class, default dict
+            The collections.abc.Mapping subclass to use as the return
+            object. Can be the actual class or an empty
+            instance of the mapping type you want.  If you want a
+            collections.defaultdict, you must pass it initialized.
+
+        Returns
+        -------
+        collections.abc.Mapping
+            Key-value representation of Series.
+
+        Examples
+        --------
+        >>> import mars.dataframe as md
+        >>> s = md.Series([1, 2, 3, 4])
+        >>> s.to_dict()
+        {0: 1, 1: 2, 2: 3, 3: 4}
+        >>> from collections import OrderedDict, defaultdict
+        >>> s.to_dict(OrderedDict)
+        OrderedDict([(0, 1), (1, 2), (2, 3), (3, 4)])
+        >>> dd = defaultdict(list)
+        >>> s.to_dict(dd)
+        defaultdict(<class 'list'>, {0: 1, 1: 2, 2: 3, 3: 4})
+        """
+        return self._data.to_dict(into=into, batch_size=batch_size, session=session)
 
     def to_frame(self, name=None):
         """
@@ -1012,6 +1250,13 @@ class BaseDataFrameData(HasShapeTileableData, _ToPandasMixin):
     def columns_value(self):
         return self._columns_value
 
+    @property
+    def empty(self):
+        shape = getattr(self, '_shape')
+        if np.any(np.isnan(shape)):
+            raise ValueError('Tileable object must be executed first')
+        return (0 in shape)
+
     def to_tensor(self, dtype=None):
         from ..tensor.datasource.from_dataframe import from_dataframe
         return from_dataframe(self, dtype=dtype)
@@ -1036,14 +1281,14 @@ class BaseDataFrameData(HasShapeTileableData, _ToPandasMixin):
     def columns(self):
         from .datasource.index import from_pandas as from_pandas_index
 
-        return from_pandas_index(self.dtypes.index)
+        return from_pandas_index(self.dtypes.index, store_data=True)
 
     @property
     def axes(self):
         return [self.index, self.columns]
 
 
-class DataFrameData(BaseDataFrameData):
+class DataFrameData(_BatchedFetcher, BaseDataFrameData):
     _type_name = 'DataFrame'
 
     def _to_str(self, representation=False):
@@ -1084,6 +1329,9 @@ class DataFrameData(BaseDataFrameData):
     def __repr__(self):
         return self._to_str(representation=True)
 
+    def __mars_tensor__(self, dtype=None, order='K'):
+        return self.to_tensor().astype(dtype=dtype, order=order, copy=False)
+
     def _repr_html_(self):
         if len(self._executed_sessions) == 0:
             # not executed before, fall back to normal repr
@@ -1114,26 +1362,19 @@ class DataFrameData(BaseDataFrameData):
             return DataFrameDef
         return super().cls(provider)
 
-    def _iter_wrap(self, method, batch_size, session, **kw):
-        from .indexing.iloc import iloc
+    def items(self):
+        for col_name in self.dtypes.index:
+            yield col_name, self[col_name]
 
-        # trigger execution
-        self.execute(session=session)
-
-        size = self.shape[0]
-        n_batch = ceildiv(size, batch_size)
-
-        for i in range(n_batch):
-            batch_data = iloc(self)[size * i: size * (i + 1)] \
-                .fetch(session=session)
-            yield from getattr(batch_data, method)(**kw)
+    iteritems = items
 
     def iterrows(self, batch_size=1000, session=None):
-        return self._iter_wrap('iterrows', batch_size, session)
+        for batch_data in self.iterbatch(batch_size=batch_size, session=session):
+            yield from getattr(batch_data, 'iterrows')()
 
     def itertuples(self, index=True, name='Pandas', batch_size=1000, session=None):
-        return self._iter_wrap('itertuples', batch_size, session,
-                               index=index, name=name)
+        for batch_data in self.iterbatch(batch_size=batch_size, session=session):
+            yield from getattr(batch_data, 'itertuples')(index=index, name=name)
 
 
 class DataFrame(HasShapeTileableEnity, _ToPandasMixin):
@@ -1153,7 +1394,7 @@ class DataFrame(HasShapeTileableEnity, _ToPandasMixin):
         return self._data.from_records(records, **kw)
 
     def __mars_tensor__(self, dtype=None, order='K'):
-        return self._data.to_tensor().astype(dtype=dtype, order=order, copy=False)
+        return self._data.__mars_tensor__(dtype=dtype, order=order)
 
     def __getattr__(self, key):
         try:
@@ -1194,19 +1435,40 @@ class DataFrame(HasShapeTileableEnity, _ToPandasMixin):
 
     @property
     def index(self):
-        return self._data.index
+        idx = self._data.index
+        idx._set_df_or_series(self, 0)
+        return idx
+
+    @index.setter
+    def index(self, new_index):
+        self.set_axis(new_index, axis=0, inplace=True)
 
     @property
     def columns(self):
-        return self._data.columns
+        col = self._data.columns
+        col._set_df_or_series(self, 1)
+        return col
 
     @columns.setter
     def columns(self, new_columns):
-        from .indexing.set_label import DataFrameSetLabel
+        self.set_axis(new_columns, axis=1, inplace=True)
 
-        op = DataFrameSetLabel(axis=1, value=new_columns)
-        new_df = op(self)
-        self.data = new_df.data
+    def keys(self):
+        """
+        Get the 'info axis' (see Indexing for more).
+
+        This is index for Series, columns for DataFrame.
+
+        Returns
+        -------
+        Index
+            Info axis.
+        """
+        return self.columns
+
+    @property
+    def values(self):
+        return self.to_tensor()
 
     @property
     def dtypes(self):
