@@ -19,23 +19,22 @@ from urllib.parse import urlparse
 
 import pandas as pd
 import numpy as np
+try:
+    from pyarrow import NativeFile
+except ImportError:  # pragma: no cover
+    NativeFile = None
 
 from ... import opcodes as OperandDef
 from ...config import options
 from ...core import OutputType
-from ...utils import parse_readable_size, lazy_import, FixedSizeFileObject
+from ...lib.filesystem import get_fs, open_file, file_size, glob
 from ...serialize import StringField, DictField, ListField, Int32Field, Int64Field, BoolField, AnyField
-from ...filesystem import get_fs, open_file, file_size, glob
+from ...utils import parse_readable_size, lazy_import, FixedSizeFileObject
 from ..arrays import ArrowStringDtype
 from ..core import IndexValue
 from ..utils import parse_index, build_empty_df, standardize_range_index, \
     to_arrow_dtypes, contain_arrow_dtype
-from ..operands import DataFrameOperand, DataFrameOperandMixin
-
-try:
-    from pyarrow import HdfsFile
-except ImportError:  # pragma: no cover
-    HdfsFile = None
+from .core import HeadOptimizedDataSource, ColumnPruneSupportedDataSourceMixin
 
 
 cudf = lazy_import('cudf', globals=globals())
@@ -68,7 +67,7 @@ def _find_hdfs_start_end(f, offset, size):
 
 
 def _find_chunk_start_end(f, offset, size):
-    if HdfsFile is not None and isinstance(f, HdfsFile):
+    if NativeFile is not None and isinstance(f, NativeFile):
         return _find_hdfs_start_end(f, offset, size)
     f.seek(offset)
     if f.tell() == 0:
@@ -82,7 +81,7 @@ def _find_chunk_start_end(f, offset, size):
     return start, end
 
 
-class DataFrameReadCSV(DataFrameOperand, DataFrameOperandMixin):
+class DataFrameReadCSV(HeadOptimizedDataSource, ColumnPruneSupportedDataSourceMixin):
     _op_type_ = OperandDef.READ_CSV
 
     _path = AnyField('path')
@@ -103,7 +102,8 @@ class DataFrameReadCSV(DataFrameOperand, DataFrameOperandMixin):
     def __init__(self, path=None, names=None, sep=None, header=None, index_col=None,
                  compression=None, usecols=None, offset=None, size=None, nrows=None,
                  gpu=None, keep_usecols_order=None, incremental_index=None,
-                 use_arrow_dtype=None, storage_options=None, **kw):
+                 use_arrow_dtype=None, storage_options=None,
+                 memory_scale=None, **kw):
         super().__init__(_path=path, _names=names, _sep=sep, _header=header,
                          _index_col=index_col, _compression=compression,
                          _usecols=usecols, _offset=offset, _size=size, _nrows=nrows,
@@ -111,7 +111,8 @@ class DataFrameReadCSV(DataFrameOperand, DataFrameOperandMixin):
                          _keep_usecols_order=keep_usecols_order,
                          _use_arrow_dtype=use_arrow_dtype,
                          _storage_options=storage_options,
-                         _output_types=[OutputType.dataframe], **kw)
+                         _output_types=[OutputType.dataframe],
+                         _memory_scale=memory_scale, **kw)
 
     @property
     def path(self):
@@ -141,10 +142,6 @@ class DataFrameReadCSV(DataFrameOperand, DataFrameOperandMixin):
     def usecols(self):
         return self._usecols
 
-    @property
-    def nrows(self):
-        return self._nrows
-
     @property
     def offset(self):
         return self._offset
@@ -169,6 +166,13 @@ class DataFrameReadCSV(DataFrameOperand, DataFrameOperandMixin):
     def storage_options(self):
         return self._storage_options
 
+    def get_columns(self):
+        return self._usecols
+
+    def set_pruned_columns(self, columns, *, keep_order=None):
+        self._usecols = columns
+        self._keep_usecols_order = keep_order
+
     @classmethod
     def _tile_compressed(cls, op):
         # Compression does not support break into small parts
@@ -187,7 +191,7 @@ class DataFrameReadCSV(DataFrameOperand, DataFrameOperandMixin):
                                      chunks=[new_chunk], nsplits=nsplits)
 
     @classmethod
-    def tile(cls, op):
+    def _tile(cls, op):
         if op.compression:
             return cls._tile_compressed(op)
 
@@ -334,6 +338,10 @@ class DataFrameReadCSV(DataFrameOperand, DataFrameOperandMixin):
 
         ctx[out_df.key] = df
 
+    def estimate_size(cls, ctx, op):
+        phy_size = op.size * (op.memory_scale or 1)
+        ctx[op.outputs[0].key] = (phy_size, phy_size * 2)
+
     def __call__(self, index_value=None, columns_value=None, dtypes=None, chunk_bytes=None):
         shape = (np.nan, len(dtypes))
         return self.new_dataframe(None, shape, dtypes=dtypes, index_value=index_value,
@@ -343,7 +351,7 @@ class DataFrameReadCSV(DataFrameOperand, DataFrameOperandMixin):
 def read_csv(path, names=None, sep=',', index_col=None, compression=None, header='infer',
              dtype=None, usecols=None, nrows=None, chunk_bytes='64M', gpu=None, head_bytes='100k',
              head_lines=None, incremental_index=False, use_arrow_dtype=None,
-             storage_options=None, **kwargs):
+             storage_options=None, memory_scale=None, **kwargs):
     r"""
     Read a comma-separated values (csv) file into DataFrame.
     Also supports optionally iterating or breaking of the file
@@ -669,7 +677,7 @@ def read_csv(path, names=None, sep=',', index_col=None, compression=None, header
     op = DataFrameReadCSV(path=path, names=names, sep=sep, header=header, index_col=index_col,
                           usecols=usecols, compression=compression, gpu=gpu,
                           incremental_index=incremental_index, use_arrow_dtype=use_arrow_dtype,
-                          storage_options=storage_options,
+                          storage_options=storage_options, memory_scale=memory_scale,
                           **kwargs)
     chunk_bytes = chunk_bytes or options.chunk_store_limit
     dtypes = mini_df.dtypes
