@@ -34,13 +34,19 @@ except ImportError:
 
 from ... import opcodes as OperandDef
 from ...config import options
-from ...filesystem import get_fs, open_file, glob
+from ...lib.filesystem import file_size, get_fs, glob, open_file
 from ...serialize import AnyField, BoolField, DictField, ListField,\
-    StringField, Int32Field, BytesField
+    StringField, Int32Field, Int64Field, BytesField
+from ...utils import is_object_dtype
 from ..arrays import ArrowStringDtype
-from ..operands import DataFrameOperandMixin, DataFrameOperand, OutputType
+from ..operands import OutputType
 from ..utils import parse_index, to_arrow_dtypes, contain_arrow_dtype, \
     standardize_range_index
+from .core import HeadOptimizedDataSource, ColumnPruneSupportedDataSourceMixin
+
+
+PARQUET_MEMORY_SCALE = 15
+STRING_FIELD_OVERHEAD = 50
 
 
 def check_engine(engine):
@@ -73,25 +79,34 @@ def get_engine(engine):
 
 
 class ParquetEngine:
+    def get_row_num(self, f):
+        raise NotImplementedError
+
     def read_dtypes(self, f, **kwargs):
         raise NotImplementedError
 
-    def read_to_pandas(self, f, columns=None,
+    def read_to_pandas(self, f, columns=None, nrows=None,
                        use_arrow_dtype=None, **kwargs):
         raise NotImplementedError
 
     def read_group_to_pandas(self, f, group_index, columns=None,
-                             use_arrow_dtype=None, **kwargs):
+                             nrows=None, use_arrow_dtype=None, **kwargs):
         raise NotImplementedError
 
 
 class ArrowEngine(ParquetEngine):
+    def get_row_num(self, f):
+        file = pq.ParquetFile(f)
+        return file.metadata.num_rows
+
     def read_dtypes(self, f, **kwargs):
         file = pq.ParquetFile(f)
         return file.schema_arrow.empty_table().to_pandas().dtypes
 
     @classmethod
-    def _table_to_pandas(cls, t, use_arrow_dtype=None):
+    def _table_to_pandas(cls, t, nrows=None, use_arrow_dtype=None):
+        if nrows is not None:
+            t = t.slice(0, nrows)
         if use_arrow_dtype:
             df = t.to_pandas(
                 types_mapper={pa.string(): ArrowStringDtype()}.get)
@@ -99,36 +114,44 @@ class ArrowEngine(ParquetEngine):
             df = t.to_pandas()
         return df
 
-    def read_to_pandas(self, f, columns=None,
+    def read_to_pandas(self, f, columns=None, nrows=None,
                        use_arrow_dtype=None, **kwargs):
         file = pq.ParquetFile(f)
         t = file.read(columns=columns, **kwargs)
-        return self._table_to_pandas(t, use_arrow_dtype=use_arrow_dtype)
+        return self._table_to_pandas(t, nrows=nrows,
+                                     use_arrow_dtype=use_arrow_dtype)
 
     def read_group_to_pandas(self, f, group_index, columns=None,
-                             use_arrow_dtype=None, **kwargs):
+                             nrows=None, use_arrow_dtype=None, **kwargs):
         file = pq.ParquetFile(f)
         t = file.read_row_group(group_index, columns=columns, **kwargs)
-        return self._table_to_pandas(t, use_arrow_dtype=use_arrow_dtype)
+        return self._table_to_pandas(t, nrows=nrows,
+                                     use_arrow_dtype=use_arrow_dtype)
 
 
 class FastpaquetEngine(ParquetEngine):
+    def get_row_num(self, f):
+        file = fastparquet.ParquetFile(f)
+        return file.count
+
     def read_dtypes(self, f, **kwargs):
         file = fastparquet.ParquetFile(f)
         dtypes_dict = file._dtypes()
         return pd.Series(dict((c, dtypes_dict[c]) for c in file.columns))
 
-    def read_to_pandas(self, f, columns=None,
+    def read_to_pandas(self, f, columns=None, nrows=None,
                        use_arrow_dtype=None, **kwargs):
         file = fastparquet.ParquetFile(f)
         df = file.to_pandas(**kwargs)
+        if nrows is not None:
+            df = df.head(nrows)
         if use_arrow_dtype:
             df = df.astype(to_arrow_dtypes(df.dtypes).to_dict())
 
         return df
 
 
-class DataFrameReadParquet(DataFrameOperand, DataFrameOperandMixin):
+class DataFrameReadParquet(HeadOptimizedDataSource, ColumnPruneSupportedDataSourceMixin):
     _op_type_ = OperandDef.READ_PARQUET
 
     _path = AnyField('path')
@@ -139,16 +162,23 @@ class DataFrameReadParquet(DataFrameOperand, DataFrameOperandMixin):
     _group_index = Int32Field('group_index')
     _read_kwargs = DictField('read_kwargs')
     _incremental_index = BoolField('incremental_index')
+    _nrows = Int64Field('nrows')
     _storage_options = DictField('storage_options')
-
     # for chunk
     _partitions = BytesField('partitions')
     _partition_keys = ListField('partition_keys')
+    _num_group_rows = Int64Field('num_group_rows')
+    # as read meta may be too time-consuming when number of files is large,
+    # thus we only read first file to get row number and raw file size
+    _first_chunk_row_num = Int64Field('first_chunk_row_num')
+    _first_chunk_raw_bytes = Int64Field('first_chunk_raw_bytes')
 
     def __init__(self, path=None, engine=None, columns=None, use_arrow_dtype=None,
                  groups_as_chunks=None, group_index=None, incremental_index=None,
                  read_kwargs=None, partitions=None, partition_keys=None,
-                 storage_options=None, **kw):
+                 num_group_rows=None, nrows=None,
+                 storage_options=None, first_chunk_row_num=None,
+                 first_chunk_raw_bytes=None, memory_scale=None, **kw):
         super().__init__(_path=path, _engine=engine, _columns=columns,
                          _use_arrow_dtype=use_arrow_dtype,
                          _groups_as_chunks=groups_as_chunks,
@@ -157,7 +187,11 @@ class DataFrameReadParquet(DataFrameOperand, DataFrameOperandMixin):
                          _incremental_index=incremental_index,
                          _partitions=partitions,
                          _partition_keys=partition_keys,
+                         _num_group_rows=num_group_rows, _nrows=nrows,
                          _storage_options=storage_options,
+                         _memory_scale=memory_scale,
+                         _first_chunk_row_num=first_chunk_row_num,
+                         _first_chunk_raw_bytes=first_chunk_raw_bytes,
                          _output_types=[OutputType.dataframe], **kw)
 
     @property
@@ -200,10 +234,28 @@ class DataFrameReadParquet(DataFrameOperand, DataFrameOperandMixin):
     def partition_keys(self):
         return self._partition_keys
 
+    @property
+    def num_group_rows(self):
+        return self._num_group_rows
+
     @property
     def storage_options(self):
         return self._storage_options
 
+    @property
+    def first_chunk_row_num(self):
+        return self._first_chunk_row_num
+
+    @property
+    def first_chunk_raw_bytes(self):
+        return self._first_chunk_raw_bytes
+
+    def get_columns(self):
+        return self._columns
+
+    def set_pruned_columns(self, columns, *, keep_order=None):
+        self._columns = columns
+
     @classmethod
     def _to_arrow_dtypes(cls, dtypes, op):
         if op.use_arrow_dtype is None and not op.gpu and \
@@ -213,7 +265,7 @@ class DataFrameReadParquet(DataFrameOperand, DataFrameOperandMixin):
         return dtypes
 
     @classmethod
-    def _tile_partitioned(cls, op):
+    def _tile_partitioned(cls, op: "DataFrameReadParquet"):
         out_df = op.outputs[0]
         shape = (np.nan, out_df.shape[1])
         dtypes = cls._to_arrow_dtypes(out_df.dtypes, op)
@@ -227,11 +279,17 @@ class DataFrameReadParquet(DataFrameOperand, DataFrameOperandMixin):
 
         chunk_index = 0
         out_chunks = []
-        for piece in dataset.pieces:
+        first_chunk_row_num, first_chunk_raw_bytes = None, None
+        for i, piece in enumerate(dataset.pieces):
             chunk_op = op.copy().reset_key()
-            chunk_op._path = path_prefix + piece.path
+            chunk_op._path = chunk_path = path_prefix + piece.path
             chunk_op._partitions = pickle.dumps(dataset.partitions)
             chunk_op._partition_keys = piece.partition_keys
+            if i == 0:
+                first_chunk_raw_bytes = file_size(chunk_path, op.storage_options)
+                first_chunk_row_num = piece.get_metadata().num_rows
+            chunk_op._first_chunk_row_num = first_chunk_row_num
+            chunk_op._first_chunk_raw_bytes = first_chunk_raw_bytes
             new_chunk = chunk_op.new_chunk(
                 None, shape=shape, index=(chunk_index, 0),
                 index_value=out_df.index_value,
@@ -248,7 +306,7 @@ class DataFrameReadParquet(DataFrameOperand, DataFrameOperandMixin):
                                      chunks=out_chunks, nsplits=nsplits)
 
     @classmethod
-    def _tile_no_partitioned(cls, op):
+    def _tile_no_partitioned(cls, op: "DataFrameReadParquet"):
         chunk_index = 0
         out_chunks = []
         out_df = op.outputs[0]
@@ -259,12 +317,22 @@ class DataFrameReadParquet(DataFrameOperand, DataFrameOperandMixin):
         paths = op.path if isinstance(op.path, (tuple, list)) else \
             glob(op.path, storage_options=op.storage_options)
 
-        for pth in paths:
+        first_chunk_row_num, first_chunk_raw_bytes = None, None
+        for i, pth in enumerate(paths):
+            if i == 0:
+                with open_file(pth, storage_options=op.storage_options) as f:
+                    first_chunk_row_num = get_engine(op.engine).get_row_num(f)
+                first_chunk_raw_bytes = file_size(pth, storage_options=op.storage_options)
+
             if op.groups_as_chunks:
-                for group_idx in range(pq.ParquetFile(pth).num_row_groups):
+                num_row_groups = pq.ParquetFile(pth).num_row_groups
+                for group_idx in range(num_row_groups):
                     chunk_op = op.copy().reset_key()
                     chunk_op._path = pth
                     chunk_op._group_index = group_idx
+                    chunk_op._first_chunk_row_num = first_chunk_row_num
+                    chunk_op._first_chunk_raw_bytes = first_chunk_raw_bytes
+                    chunk_op._num_group_rows = num_row_groups
                     new_chunk = chunk_op.new_chunk(
                         None, shape=shape, index=(chunk_index, 0),
                         index_value=out_df.index_value,
@@ -275,6 +343,8 @@ class DataFrameReadParquet(DataFrameOperand, DataFrameOperandMixin):
             else:
                 chunk_op = op.copy().reset_key()
                 chunk_op._path = pth
+                chunk_op._first_chunk_row_num = first_chunk_row_num
+                chunk_op._first_chunk_raw_bytes = first_chunk_raw_bytes
                 new_chunk = chunk_op.new_chunk(
                     None, shape=shape, index=(chunk_index, 0),
                     index_value=out_df.index_value,
@@ -294,23 +364,25 @@ class DataFrameReadParquet(DataFrameOperand, DataFrameOperandMixin):
                                      chunks=out_chunks, nsplits=nsplits)
 
     @classmethod
-    def tile(cls, op):
+    def _tile(cls, op):
         if get_fs(op.path, op.storage_options).isdir(op.path):
             return cls._tile_partitioned(op)
         else:
             return cls._tile_no_partitioned(op)
 
     @classmethod
-    def _execute_partitioned(cls, ctx, op):
+    def _execute_partitioned(cls, ctx, op: "DataFrameReadParquet"):
         out = op.outputs[0]
         partitions = pickle.loads(op.partitions)
         piece = pq.ParquetDatasetPiece(op.path, partition_keys=op.partition_keys,
                                        open_file_func=open_file)
-        df = piece.read(partitions=partitions).to_pandas()
-        ctx[out.key] = df
+        table = piece.read(partitions=partitions)
+        if op.nrows is not None:
+            table = table.slice(0, op.nrows)
+        ctx[out.key] = table.to_pandas()
 
     @classmethod
-    def execute(cls, ctx, op):
+    def execute(cls, ctx, op: "DataFrameReadParquet"):
         out = op.outputs[0]
         path = op.path
 
@@ -322,15 +394,34 @@ class DataFrameReadParquet(DataFrameOperand, DataFrameOperandMixin):
             use_arrow_dtype = contain_arrow_dtype(out.dtypes)
             if op.groups_as_chunks:
                 df = engine.read_group_to_pandas(f, op.group_index, columns=op.columns,
+                                                 nrows=op.nrows,
                                                  use_arrow_dtype=use_arrow_dtype,
                                                  **op.read_kwargs or dict())
             else:
                 df = engine.read_to_pandas(f, columns=op.columns,
+                                           nrows=op.nrows,
                                            use_arrow_dtype=use_arrow_dtype,
                                            **op.read_kwargs or dict())
 
             ctx[out.key] = df
 
+    @classmethod
+    def estimate_size(cls, ctx, op: "DataFrameReadParquet"):
+        first_chunk_row_num = op.first_chunk_row_num
+        first_chunk_raw_bytes = op.first_chunk_raw_bytes
+        raw_bytes = file_size(op.path, storage_options=op.storage_options)
+        if op.num_group_rows:
+            raw_bytes = np.ceil(np.divide(raw_bytes, op.num_row_groups)) \
+                .astype(np.int64).item()
+
+        estimated_row_num = np.ceil(
+            first_chunk_row_num * (raw_bytes / first_chunk_raw_bytes))\
+            .astype(np.int64).item()
+        phy_size = raw_bytes * (op.memory_scale or PARQUET_MEMORY_SCALE)
+        n_strings = len([dt for dt in op.outputs[0].dtypes if is_object_dtype(dt)])
+        pd_size = phy_size + n_strings * estimated_row_num * STRING_FIELD_OVERHEAD
+        ctx[op.outputs[0].key] = (pd_size, pd_size + phy_size)
+
     def __call__(self, index_value=None, columns_value=None, dtypes=None):
         shape = (np.nan, len(dtypes))
         return self.new_dataframe(None, shape, dtypes=dtypes, index_value=index_value,
@@ -340,7 +431,7 @@ class DataFrameReadParquet(DataFrameOperand, DataFrameOperandMixin):
 def read_parquet(path, engine: str = "auto", columns=None,
                  groups_as_chunks=False, use_arrow_dtype=None,
                  incremental_index=False, storage_options=None,
-                 **kwargs):
+                 memory_scale=None, **kwargs):
     """
     Load a parquet object from the file path, returning a DataFrame.
 
@@ -372,6 +463,8 @@ def read_parquet(path, engine: str = "auto", columns=None,
         If True, use arrow dtype to store columns.
     storage_options: dict, optional
         Options for storage connection.
+    memory_scale: int, optional
+        Scale that real memory occupation divided with raw file size.
     **kwargs
         Any additional kwargs are passed to the engine.
 
@@ -416,6 +509,7 @@ def read_parquet(path, engine: str = "auto", columns=None,
                               use_arrow_dtype=use_arrow_dtype,
                               read_kwargs=kwargs,
                               incremental_index=incremental_index,
-                              storage_options=storage_options)
+                              storage_options=storage_options,
+                              memory_scale=memory_scale)
     return op(index_value=index_value, columns_value=columns_value,
               dtypes=dtypes)
