@@ -28,12 +28,13 @@ from ...serialize import StringField, AnyField, BoolField, ListField, \
 from ...tensor.utils import normalize_chunk_sizes
 from ..arrays import ArrowStringDtype
 from ..core import IndexValue
-from ..operands import DataFrameOperand, DataFrameOperandMixin, OutputType
+from ..operands import DataFrameOperand, OutputType
 from ..utils import parse_index, create_sa_connection, \
     standardize_range_index, to_arrow_dtypes
+from .core import ColumnPruneSupportedDataSourceMixin
 
 
-class DataFrameReadSQL(DataFrameOperand, DataFrameOperandMixin):
+class DataFrameReadSQL(DataFrameOperand, ColumnPruneSupportedDataSourceMixin):
     _op_type_ = OperandDef.READ_SQL
 
     _table_or_sql = StringField('table_or_sql')
@@ -164,6 +165,12 @@ class DataFrameReadSQL(DataFrameOperand, DataFrameOperandMixin):
     def nrows(self):
         return self._nrows
 
+    def get_columns(self):
+        return self._columns
+
+    def set_pruned_columns(self, columns, *, keep_order=None):
+        self._columns = columns
+
     def _get_selectable(self, engine_or_conn, columns=None):
         import sqlalchemy as sa
         from sqlalchemy import sql
@@ -483,7 +490,11 @@ class DataFrameReadSQL(DataFrameOperand, DataFrameOperandMixin):
                     if isinstance(dtype, ArrowStringDtype):
                         df.iloc[:, i] = df.iloc[:, i].astype(dtype)
 
-            ctx[out.key] = df
+            if out.ndim == 2:
+                ctx[out.key] = df
+            else:
+                # this happens when column pruning results in one single series
+                ctx[out.key] = df.iloc[:, 0]
         finally:
             engine.dispose()
 
