@@ -12,6 +12,7 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+import functools
 import itertools
 from typing import List, Dict
 
@@ -31,8 +32,7 @@ from ..merge import DataFrameConcat
 from ..operands import DataFrameOperand, DataFrameOperandMixin, DataFrameShuffleProxy
 from ..reduction.core import ReductionCompiler, ReductionSteps, ReductionPreStep, \
     ReductionAggStep, ReductionPostStep
-from ..reduction.aggregation import mean_function, var_function, sem_function, \
-    skew_function, kurt_function, is_funcs_aggregate, normalize_reduction_funcs
+from ..reduction.aggregation import is_funcs_aggregate, normalize_reduction_funcs
 from ..utils import parse_index, build_concatenated_rows_frame
 from .core import DataFrameGroupByOperand
 
@@ -50,13 +50,13 @@ _agg_functions = {
     'any': lambda x: x.any(),
     'count': lambda x: x.count(),
     'size': lambda x: x._reduction_size(),
-    'mean': mean_function,
-    'var': var_function,
-    'std': lambda x, ddof=1: var_function(x, ddof=ddof) ** 0.5,
-    'sem': sem_function,
-    'skew': skew_function,
-    'kurt': kurt_function,
-    'kurtosis': kurt_function,
+    'mean': lambda x: x.mean(),
+    'var': lambda x, ddof=1: x.var(ddof=ddof),
+    'std': lambda x, ddof=1: x.std(ddof=ddof),
+    'sem': lambda x, ddof=1: x.sem(ddof=ddof),
+    'skew': lambda x, bias=False: x.skew(bias=bias),
+    'kurt': lambda x, bias=False: x.kurt(bias=bias),
+    'kurtosis': lambda x, bias=False: x.kurtosis(bias=bias),
 }
 _series_col_name = 'col_name'
 
@@ -65,14 +65,17 @@ def _patch_groupby_kurt():
     try:
         from pandas.core.groupby import DataFrameGroupBy, SeriesGroupBy
         if not hasattr(DataFrameGroupBy, 'kurt'):  # pragma: no branch
-            DataFrameGroupBy.kurt = lambda x: x.agg(pd.Series.kurt)
-            SeriesGroupBy.kurt = lambda x: x.agg(pd.Series.kurt)
+            def _kurt_by_frame(a, *args, **kwargs):
+                return a.to_frame().kurt(*args, **kwargs).iloc[0]
 
-            def kurtosis(x, *args, **kwargs):
-                return pd.Series.kurt(x, *args, **kwargs)
+            def _group_kurt(x, *args, **kwargs):
+                if kwargs.get('numeric_only') is not None:
+                    return x.agg(functools.partial(_kurt_by_frame, *args, **kwargs))
+                else:
+                    return x.agg(functools.partial(pd.Series.kurt, *args, **kwargs))
 
-            DataFrameGroupBy.kurtosis = lambda x: x.agg(kurtosis)
-            SeriesGroupBy.kurtosis = lambda x: x.agg(kurtosis)
+            DataFrameGroupBy.kurt = DataFrameGroupBy.kurtosis = _group_kurt
+            SeriesGroupBy.kurt = SeriesGroupBy.kurtosis = _group_kurt
     except (AttributeError, ImportError):  # pragma: no cover
         pass
 
@@ -99,19 +102,20 @@ class DataFrameGroupByAgg(DataFrameOperand, DataFrameOperandMixin):
     _pre_funcs = ListField('pre_funcs')
     _agg_funcs = ListField('agg_funcs')
     _post_funcs = ListField('post_funcs')
+    _index_levels = Int32Field('index_levels')
     # for chunk
     _tileable_op_key = StringField('tileable_op_key')
 
     def __init__(self, raw_func=None, raw_func_kw=None, func=None, func_rename=None,
                  method=None, groupby_params=None, use_inf_as_na=None, combine_size=None,
                  pre_funcs=None, agg_funcs=None, post_funcs=None, stage=None,
-                 output_types=None, tileable_op_key=None, **kw):
+                 output_types=None, tileable_op_key=None, index_levels=None, **kw):
         super().__init__(_raw_func=raw_func, _raw_func_kw=raw_func_kw, _func=func,
                          _func_rename=func_rename, _method=method, _groupby_params=groupby_params,
                          _combine_size=combine_size, _use_inf_as_na=use_inf_as_na,
                          _pre_funcs=pre_funcs, _agg_funcs=agg_funcs, _post_funcs=post_funcs,
                          _stage=stage, _output_types=output_types,
-                         _tileable_op_key=tileable_op_key, **kw)
+                         _tileable_op_key=tileable_op_key, _index_levels=index_levels, **kw)
 
     @property
     def raw_func(self):
@@ -157,6 +161,10 @@ class DataFrameGroupByAgg(DataFrameOperand, DataFrameOperandMixin):
     def post_funcs(self) -> List[ReductionPostStep]:
         return self._post_funcs
 
+    @property
+    def index_levels(self) -> int:
+        return self._index_levels
+
     @property
     def tileable_op_key(self):
         return self._tileable_op_key
@@ -189,14 +197,22 @@ class DataFrameGroupByAgg(DataFrameOperand, DataFrameOperandMixin):
 
         as_index = self.groupby_params.get('as_index')
         # make sure if as_index=False takes effect
+        if isinstance(agg_df.index, pd.MultiIndex):
+            # if MultiIndex, as_index=False definitely takes no effect
+            self.groupby_params['as_index'] = as_index = True
+        elif agg_df.index.name is not None:
+            # if not MultiIndex and agg_df.index has a name
+            # means as_index=False takes no effect
+            self.groupby_params['as_index'] = as_index = True
+
+        # determine num of indices to group in intermediate steps
         if not as_index:
-            if isinstance(agg_df.index, pd.MultiIndex):
-                # if MultiIndex, as_index=False definitely takes no effect
-                self.groupby_params['as_index'] = True
-            elif agg_df.index.name is not None:
-                # if not MultiIndex and agg_df.index has a name
-                # means as_index=False takes no effect
-                self.groupby_params['as_index'] = True
+            as_index_agg_df = groupby.op.build_mock_groupby(as_index=True) \
+                .aggregate(self.raw_func, **self.raw_func_kw)
+            pd_index = as_index_agg_df.index
+        else:
+            pd_index = agg_df.index
+        self._index_levels = 1 if not isinstance(pd_index, pd.MultiIndex) else len(pd_index.levels)
 
         inputs = self._get_inputs([input_df])
         return self.new_dataframe(inputs, shape=shape, dtypes=agg_df.dtypes,
@@ -210,6 +226,11 @@ class DataFrameGroupByAgg(DataFrameOperand, DataFrameOperandMixin):
         index_value.value.should_be_monotonic = True
 
         inputs = self._get_inputs([in_series])
+
+        # determine num of indices to group in intermediate steps
+        pd_index = agg_result.index
+        self._index_levels = 1 if not isinstance(pd_index, pd.MultiIndex) else len(pd_index.levels)
+
         # update value type
         if isinstance(agg_result, pd.DataFrame):
             return self.new_dataframe(inputs, shape=(np.nan, len(agg_result.columns)),
@@ -265,37 +286,37 @@ class DataFrameGroupByAgg(DataFrameOperand, DataFrameOperandMixin):
 
     @classmethod
     def _gen_map_chunks(cls, op: "DataFrameGroupByAgg", in_df, out_df, func_infos: ReductionSteps):
-        agg_chunks = []
+        map_chunks = []
         for chunk in in_df.chunks:
             chunk_inputs = [chunk]
-            agg_op = op.copy().reset_key()
+            map_op = op.copy().reset_key()
             # force as_index=True for map phase
-            agg_op.output_types = [OutputType.dataframe]
-            agg_op._groupby_params = agg_op.groupby_params.copy()
-            agg_op._groupby_params['as_index'] = True
-            if isinstance(agg_op._groupby_params['by'], list):
+            map_op.output_types = [OutputType.dataframe]
+            map_op._groupby_params = map_op.groupby_params.copy()
+            map_op._groupby_params['as_index'] = True
+            if isinstance(map_op._groupby_params['by'], list):
                 by = []
-                for v in agg_op._groupby_params['by']:
+                for v in map_op._groupby_params['by']:
                     if isinstance(v, (Base, Entity)):
                         by_chunk = v.cix[chunk.index[0], ]
                         chunk_inputs.append(by_chunk)
                         by.append(by_chunk)
                     else:
                         by.append(v)
-                agg_op._groupby_params['by'] = by
-            agg_op._stage = OperandStage.map
-            agg_op._pre_funcs = func_infos.pre_funcs
-            agg_op._agg_funcs = func_infos.agg_funcs
+                map_op._groupby_params['by'] = by
+            map_op._stage = OperandStage.map
+            map_op._pre_funcs = func_infos.pre_funcs
+            map_op._agg_funcs = func_infos.agg_funcs
             new_index = chunk.index if len(chunk.index) == 2 else (chunk.index[0], 0)
             if op.output_types[0] == OutputType.dataframe:
-                agg_chunk = agg_op.new_chunk(chunk_inputs, shape=out_df.shape, index=new_index,
+                map_chunk = map_op.new_chunk(chunk_inputs, shape=out_df.shape, index=new_index,
                                              index_value=out_df.index_value,
                                              columns_value=out_df.columns_value)
             else:
-                agg_chunk = agg_op.new_chunk(chunk_inputs, shape=(out_df.shape[0], 1), index=new_index,
+                map_chunk = map_op.new_chunk(chunk_inputs, shape=(out_df.shape[0], 1), index=new_index,
                                              index_value=out_df.index_value)
-            agg_chunks.append(agg_chunk)
-        return agg_chunks
+            map_chunks.append(map_chunk)
+        return map_chunks
 
     @classmethod
     def _compile_funcs(cls, op: "DataFrameGroupByAgg", in_df) -> ReductionSteps:
@@ -326,9 +347,6 @@ class DataFrameGroupByAgg(DataFrameOperand, DataFrameOperandMixin):
             in_df = build_concatenated_rows_frame(in_df)
         out_df = op.outputs[0]
 
-        index = out_df.index_value.to_pandas()
-        level = 0 if not isinstance(index, pd.MultiIndex) else list(range(len(index.levels)))
-
         func_infos = cls._compile_funcs(op, in_df)
 
         # First, perform groupby and aggregation on each chunk.
@@ -346,7 +364,7 @@ class DataFrameGroupByAgg(DataFrameOperand, DataFrameOperandMixin):
             agg_op._groupby_params.pop('selection', None)
             # use levels instead of by for reducer
             agg_op._groupby_params.pop('by', None)
-            agg_op._groupby_params['level'] = level
+            agg_op._groupby_params['level'] = list(range(op.index_levels))
             agg_op._stage = OperandStage.agg
             agg_op._agg_funcs = func_infos.agg_funcs
             agg_op._post_funcs = func_infos.post_funcs
@@ -376,9 +394,6 @@ class DataFrameGroupByAgg(DataFrameOperand, DataFrameOperandMixin):
             in_df = build_concatenated_rows_frame(in_df)
         out_df = op.outputs[0]
 
-        index = out_df.index_value.to_pandas()
-        level = 0 if not isinstance(index, pd.MultiIndex) else list(range(len(index.levels)))
-
         func_infos = cls._compile_funcs(op, in_df)
         combine_size = op.combine_size
         chunks = cls._gen_map_chunks(op, in_df, out_df, func_infos)
@@ -402,7 +417,7 @@ class DataFrameGroupByAgg(DataFrameOperand, DataFrameOperandMixin):
                 chunk_op._groupby_params.pop('selection', None)
                 # use levels instead of by for agg
                 chunk_op._groupby_params.pop('by', None)
-                chunk_op._groupby_params['level'] = level
+                chunk_op._groupby_params['level'] = list(range(op.index_levels))
                 chunk_op._agg_funcs = func_infos.agg_funcs
 
                 new_shape = (np.nan, out_df.shape[1]) if len(out_df.shape) == 2 else (np.nan,)
@@ -421,7 +436,7 @@ class DataFrameGroupByAgg(DataFrameOperand, DataFrameOperandMixin):
         chunk_op._groupby_params.pop('selection', None)
         # use levels instead of by for agg
         chunk_op._groupby_params.pop('by', None)
-        chunk_op._groupby_params['level'] = level
+        chunk_op._groupby_params['level'] = list(range(op.index_levels))
         chunk_op._agg_funcs = func_infos.agg_funcs
         chunk_op._post_funcs = func_infos.post_funcs
         kw = out_df.params.copy()
@@ -467,7 +482,7 @@ class DataFrameGroupByAgg(DataFrameOperand, DataFrameOperandMixin):
         elif isinstance(params.get('by'), list):
             new_by = []
             for v in params['by']:
-                if isinstance(v, Base):
+                if isinstance(v, (Base, Entity)):
                     new_by.append(ctx[v.key])
                 else:
                     new_by.append(v)
@@ -480,7 +495,7 @@ class DataFrameGroupByAgg(DataFrameOperand, DataFrameOperandMixin):
             params['sort'] = False
             grouped = df.groupby(**params)
 
-        if selection:
+        if selection is not None:
             grouped = grouped[selection]
         return grouped
 
@@ -642,8 +657,8 @@ class DataFrameGroupByAgg(DataFrameOperand, DataFrameOperandMixin):
     @classmethod
     def _execute_agg(cls, ctx, op: "DataFrameGroupByAgg"):
         xdf = cudf if op.gpu else pd
-        out = op.outputs[0]
-        col_value = out.columns_value.to_pandas() if hasattr(out, 'columns_value') else None
+        out_chunk = op.outputs[0]
+        col_value = out_chunk.columns_value.to_pandas() if hasattr(out_chunk, 'columns_value') else None
 
         in_data_tuple = ctx[op.inputs[0].key]
         in_data_list = []
@@ -655,7 +670,7 @@ class DataFrameGroupByAgg(DataFrameOperand, DataFrameOperandMixin):
         in_data_dict = cls._pack_inputs(op.agg_funcs, in_data_tuple)
 
         for _input_key, _map_func_name, agg_func_name, custom_reduction, \
-            output_key, _output_limit, kwds in op.agg_funcs:
+                output_key, _output_limit, kwds in op.agg_funcs:
             if agg_func_name == 'custom_reduction':
                 input_obj = tuple(cls._get_grouped(op, o, ctx) for o in in_data_dict[output_key])
                 in_data_dict[output_key] = cls._do_custom_agg(op, custom_reduction, *input_obj)[0]
@@ -670,12 +685,18 @@ class DataFrameGroupByAgg(DataFrameOperand, DataFrameOperandMixin):
             else:
                 func_inputs = [in_data_dict[k][cols] for k in input_keys]
 
+            if func_inputs[0].ndim == 2 and len(set(inp.shape[1] for inp in func_inputs)) > 1:
+                common_cols = func_inputs[0].columns
+                for inp in func_inputs[1:]:
+                    common_cols = common_cols.join(inp.columns, how='inner')
+                func_inputs = [inp[common_cols] for inp in func_inputs]
+
             agg_df = func(*func_inputs, gpu=op.is_gpu())
             if isinstance(agg_df, np.ndarray):
                 agg_df = xdf.DataFrame(agg_df, index=func_inputs[0].index)
 
             new_cols = None
-            if out.ndim == 2 and col_value is not None:
+            if out_chunk.ndim == 2 and col_value is not None:
                 if col_value.nlevels > agg_df.columns.nlevels:
                     new_cols = xdf.MultiIndex.from_product([agg_df.columns, [func_name]])
                 elif agg_df.shape[-1] == 1 and func_name in col_value:
@@ -687,18 +708,22 @@ class DataFrameGroupByAgg(DataFrameOperand, DataFrameOperandMixin):
                 agg_df.columns = new_cols
         aggs = [item[0] for item in aggs]
 
-        if out.ndim == 2:
+        if out_chunk.ndim == 2:
             result = xdf.concat(aggs, axis=1)
             if not op.groupby_params.get('as_index', True) \
                     and col_value.nlevels == result.columns.nlevels:
                 result.reset_index(inplace=True, drop=result.index.name in result.columns)
             result = result.reindex(col_value, axis=1)
+
+            if result.ndim == 2 and len(result) == 0:
+                result = result.astype(out_chunk.dtypes)
         else:
             result = xdf.concat(aggs)
             if result.ndim == 2:
                 result = result.iloc[:, 0]
-            result.name = out.name
-        ctx[op.outputs[0].key] = result
+            result.name = out_chunk.name
+
+        ctx[out_chunk.key] = result
 
     @classmethod
     @redirect_custom_log
