@@ -97,19 +97,9 @@ class DataFrameGroupByOperand(DataFrameMapReduceOperand, DataFrameOperandMixin):
     def build_mock_groupby(self, **kwargs):
         in_df = self.inputs[0]
         if self.is_dataframe_obj:
-            mock_obj = pd.concat([
-                build_df(in_df, size=2, fill_value=1),
-                build_df(in_df, size=2, fill_value=2),
-            ])
-            obj_dtypes = in_df.dtypes[in_df.dtypes == np.dtype('O')]
-            mock_obj[obj_dtypes.index] = mock_obj[obj_dtypes.index].radd('O')
+            mock_obj = build_df(in_df, size=[2, 2], fill_value=[1, 2], ensure_string=True)
         else:
-            mock_obj = pd.concat([
-                build_series(in_df, size=2, fill_value=1, name=in_df.name),
-                build_series(in_df, size=2, fill_value=2, name=in_df.name),
-            ])
-            if in_df.dtype == np.dtype('O'):
-                mock_obj = mock_obj.radd('O')
+            mock_obj = build_series(in_df, size=[2, 2], fill_value=[1, 2], name=in_df.name, ensure_string=True)
 
         new_kw = self.groupby_params
         new_kw.update(kwargs)
@@ -149,16 +139,14 @@ class DataFrameGroupByOperand(DataFrameMapReduceOperand, DataFrameOperandMixin):
             if isinstance(self.by, list):
                 index, types = [], []
                 for k in self.by:
-                    if isinstance(k, str):
-                        if k not in df.dtypes:
-                            raise KeyError(k)
-                        else:
-                            index.append(k)
-                            types.append(df.dtypes[k])
-                    else:
-                        assert isinstance(k, SERIES_TYPE)
+                    if isinstance(k, SERIES_TYPE):
                         index.append(k.name)
                         types.append(k.dtype)
+                    elif k in df.dtypes:
+                        index.append(k)
+                        types.append(df.dtypes[k])
+                    else:
+                        raise KeyError(k)
                 params['key_dtypes'] = pd.Series(types, index=index)
 
         inputs = [df]
@@ -178,7 +166,7 @@ class DataFrameGroupByOperand(DataFrameMapReduceOperand, DataFrameOperandMixin):
         inp_params = inp.params
         inp_params['chunks'] = inp_chunks
         inp_params['nsplits'] = nsplits
-        inp = op.copy().new_tileable(op.inputs, kws=[inp_params])
+        inp = inp.op.copy().new_tileable(op.inputs, kws=[inp_params])
 
         by_params = by.params
         by_params['chunks'] = by_chunks
@@ -228,6 +216,7 @@ class DataFrameGroupByOperand(DataFrameMapReduceOperand, DataFrameOperandMixin):
             map_op = op.copy().reset_key()
             map_op._stage = OperandStage.map
             map_op._shuffle_size = chunk_shape[0]
+            map_op._output_types = [output_type]
             chunk_inputs = [chunk]
             if len(op.inputs) > 1:
                 chunk_by = []
@@ -249,6 +238,7 @@ class DataFrameGroupByOperand(DataFrameMapReduceOperand, DataFrameOperandMixin):
             shuffle_key = ','.join(str(idx) for idx in out_idx)
             reduce_op = op.copy().reset_key()
             reduce_op._by = None
+            reduce_op._output_types = [output_type]
             reduce_op._stage = OperandStage.reduce
             reduce_op._shuffle_key = shuffle_key
             reduce_chunks.append(
@@ -294,7 +284,7 @@ class DataFrameGroupByOperand(DataFrameMapReduceOperand, DataFrameOperandMixin):
         if isinstance(by, list):
             new_by = []
             for v in by:
-                if isinstance(v, Base):
+                if isinstance(v, (Base, Entity)):
                     deliver_by = True
                     new_by.append(ctx[v.key])
                 else:
@@ -312,7 +302,7 @@ class DataFrameGroupByOperand(DataFrameMapReduceOperand, DataFrameOperandMixin):
             filters = hash_dataframe_on(df, on, op.shuffle_size, level=op.level)
 
         def _take_index(src, f):
-            result = src.loc[f]
+            result = src.iloc[f]
             if src.index.names:
                 result.index.names = src.index.names
             return result
@@ -420,10 +410,12 @@ def groupby(df, by=None, level=None, as_index=True, sort=True, group_keys=True):
         raise TypeError('as_index=False only valid with DataFrame')
 
     output_types = [OutputType.dataframe_groupby] if df.ndim == 2 else [OutputType.series_groupby]
-    if isinstance(by, (str, SERIES_TYPE, pd.Series)):
+    if isinstance(by, (SERIES_TYPE, pd.Series)):
         if isinstance(by, pd.Series):
             by = asseries(by)
         by = [by]
+    elif df.ndim > 1 and by is not None and not isinstance(by, list):
+        by = [by]
     op = DataFrameGroupByOperand(by=by, level=level, as_index=as_index, sort=sort,
                                  group_keys=group_keys, output_types=output_types)
     return op(df)
