@@ -19,9 +19,9 @@ from ... import opcodes
 from ...core import OutputType
 from ...custom_log import redirect_custom_log
 from ...serialize import BoolField, TupleField, DictField, AnyField, StringField
-from ...utils import enter_current_session
+from ...utils import enter_current_session, quiet_stdio
 from ..operands import DataFrameOperandMixin, DataFrameOperand
-from ..utils import build_empty_df, build_empty_series, parse_index, quiet_stdio
+from ..utils import build_empty_df, build_empty_series, parse_index
 
 
 class GroupByTransform(DataFrameOperand, DataFrameOperandMixin):
@@ -157,18 +157,25 @@ class GroupByTransform(DataFrameOperand, DataFrameOperandMixin):
             if op.output_types[0] == OutputType.dataframe:
                 ctx[op.outputs[0].key] = build_empty_df(out_chunk.dtypes)
             else:
-                ctx[op.outputs[0].key] = build_empty_series(out_chunk.dtype)
+                ctx[op.outputs[0].key] = build_empty_series(
+                    out_chunk.dtype, name=out_chunk.name)
             return
 
         if op.call_agg:
             result = in_data.agg(op.func, *op.args, **op.kwds)
-        else:
+        elif in_data.shape[0] > 0:
+            # cannot perform groupby-transform over empty dataframe
             result = in_data.transform(op.func, *op.args, **op.kwds)
+        else:
+            if out_chunk.ndim == 2:
+                result = pd.DataFrame(columns=out_chunk.dtypes.index)
+            else:
+                result = pd.Series([], name=out_chunk.name, dtype=out_chunk.dtype)
 
         if result.ndim == 2:
-            result = result.astype(op.outputs[0].dtypes, copy=False)
+            result = result.astype(out_chunk.dtypes, copy=False)
         else:
-            result = result.astype(op.outputs[0].dtype, copy=False)
+            result = result.astype(out_chunk.dtype, copy=False)
         ctx[op.outputs[0].key] = result
 
 
