@@ -134,8 +134,9 @@ class DataFrameIndexHandlerContext(IndexHandlerContext):
             chunk_op.output_types = [OutputType.scalar]
             kw['dtype'] = self.op.outputs[0].dtype
         elif len(shape) == 1:
-            # Series
-            chunk_op.output_types = [OutputType.series]
+            # Series or Index
+            chunk_op.output_types = [OutputType.index] if chunk_op._op_module_ == 'index' \
+                else [OutputType.series]
             kw['index_value'] = index_values[0]
             kw['dtype'] = self.op.outputs[0].dtype
             kw['name'] = getattr(self.op.outputs[0], 'name', None)
@@ -998,8 +999,12 @@ class LabelTensorFancyIndexHandler(_LabelFancyIndexHandler):
                 params['index_value'] = \
                     parse_index(out.index_value.to_pandas()[slc],
                                 slice_chunks, store_data=False)
-                params['columns_value'] = select_chunks[0].columns_value
-                params['dtypes'] = select_chunks[0].dtypes
+                if select_chunks[0].ndim == 2:
+                    params['columns_value'] = select_chunks[0].columns_value
+                    params['dtypes'] = select_chunks[0].dtypes
+                else:
+                    params['dtype'] = select_chunks[0].dtype
+                    params['name'] = select_chunks[0].name
                 params['shape'] = slice_chunks[0].shape
                 out_chunk = chunk_op.new_chunk(slice_chunks, kws=[params])
                 new_out_chunks.append(out_chunk)
