@@ -17,6 +17,7 @@ import functools
 import itertools
 from collections import OrderedDict
 from collections.abc import Iterable
+from distutils.version import LooseVersion
 from typing import List, Dict, Union
 
 import numpy as np
@@ -29,6 +30,7 @@ from ...custom_log import redirect_custom_log
 from ...operands import OperandStage
 from ...serialize import BoolField, AnyField, Int32Field, ListField, DictField
 from ...utils import ceildiv, lazy_import, enter_mode, enter_current_session
+from ..core import INDEX_CHUNK_TYPE
 from ..merge import DataFrameConcat
 from ..operands import DataFrameOperand, DataFrameOperandMixin
 from ..utils import build_df, build_empty_df, build_series, parse_index, validate_axis
@@ -38,6 +40,8 @@ from .core import CustomReduction, ReductionCompiler, ReductionSteps, ReductionP
 cp = lazy_import('cupy', globals=globals(), rename='cp')
 cudf = lazy_import('cudf', globals=globals())
 
+_agg_size_as_series = LooseVersion(pd.__version__) >= '1.3.0'
+
 
 def where_function(cond, var1, var2):
     if var1.ndim >= 1:
@@ -49,53 +53,6 @@ def where_function(cond, var1, var2):
             return np.where(cond, var1, var2).item()
 
 
-def mean_function(x, skipna=None):
-    return x.sum(skipna=skipna) / x.count()
-
-
-def var_function(x, skipna=None, ddof=1):
-    cnt = x.count()
-    if ddof == 0:
-        return (x ** 2).sum(skipna=skipna) / cnt - (x.sum(skipna=skipna) / cnt) ** 2
-    return ((x ** 2).sum(skipna=skipna) - x.sum(skipna=skipna) ** 2 / cnt) / (cnt - ddof)
-
-
-def sem_function(x, skipna=None, ddof=1):
-    var = var_function(x, skipna=skipna, ddof=ddof)
-    cnt = x.count()
-    return (var / cnt) ** 0.5
-
-
-def skew_function(x, skipna=None, bias=False):
-    cnt = x.count()
-    mean = x.sum(skipna=skipna) / cnt
-    divided = (x ** 3).sum(skipna=skipna) / cnt \
-        - 3 * (x ** 2).sum(skipna=skipna) / cnt * mean \
-        + 2 * mean ** 3
-    var = var_function(x, skipna=skipna, ddof=0)
-    val = where_function(var > 0, divided / var ** 1.5, np.nan)
-    if not bias:
-        val = where_function((var > 0) & (cnt > 2), val * ((cnt * (cnt - 1)) ** 0.5 / (cnt - 2)), np.nan)
-    return val
-
-
-def kurt_function(x, skipna=None, bias=False, fisher=True):
-    cnt = x.count()
-    mean = x.sum(skipna=skipna) / cnt
-    divided = (x ** 4).sum(skipna=skipna) / cnt \
-        - 4 * (x ** 3).sum(skipna=skipna) / cnt * mean \
-        + 6 * (x ** 2).sum(skipna=skipna) / cnt * mean ** 2 \
-        - 3 * mean ** 4
-    var = var_function(x, skipna=skipna, ddof=0)
-    val = where_function(var > 0, divided / var ** 2, np.nan)
-    if not bias:
-        val = where_function((var > 0) & (cnt > 3),
-                             (val * (cnt ** 2 - 1) - 3 * (cnt - 1) ** 2) / (cnt - 2) / (cnt - 3), np.nan)
-    if not fisher:
-        val += 3
-    return val
-
-
 _agg_functions = {
     'sum': lambda x, skipna=None: x.sum(skipna=skipna),
     'prod': lambda x, skipna=None: x.prod(skipna=skipna),
@@ -106,13 +63,13 @@ _agg_functions = {
     'any': lambda x, skipna=None: x.any(skipna=skipna),
     'count': lambda x: x.count(),
     'size': lambda x: x._reduction_size(),
-    'mean': mean_function,
-    'var': var_function,
-    'std': lambda x, skipna=None, ddof=1: var_function(x, skipna=skipna, ddof=ddof) ** 0.5,
-    'sem': sem_function,
-    'skew': skew_function,
-    'kurt': kurt_function,
-    'kurtosis': kurt_function,
+    'mean': lambda x, skipna=None: x.mean(skipna=skipna),
+    'var': lambda x, skipna=None, ddof=1: x.var(skipna=skipna, ddof=ddof),
+    'std': lambda x, skipna=None, ddof=1: x.std(skipna=skipna, ddof=ddof),
+    'sem': lambda x, skipna=None, ddof=1: x.sem(skipna=skipna, ddof=ddof),
+    'skew': lambda x, skipna=None, bias=False: x.skew(skipna=skipna, bias=bias),
+    'kurt': lambda x, skipna=None, bias=False: x.kurt(skipna=skipna, bias=bias),
+    'kurtosis': lambda x, skipna=None, bias=False: x.kurtosis(skipna=skipna, bias=bias),
 }
 
 
@@ -206,19 +163,9 @@ class DataFrameAggregate(DataFrameOperand, DataFrameOperandMixin):
                 df = df.select_dtypes([np.bool_])
 
         if self.output_types[0] == OutputType.dataframe:
-            test_obj = pd.concat([
-                build_df(df, size=2, fill_value=1),
-                build_df(df, size=2, fill_value=2),
-            ])
-            obj_dtypes = df.dtypes[df.dtypes == np.dtype('O')]
-            test_obj[obj_dtypes.index] = test_obj[obj_dtypes.index].radd('O')
+            test_obj = build_df(df, size=[2, 2], fill_value=[1, 2], ensure_string=True)
         else:
-            test_obj = pd.concat([
-                build_series(df, size=2, fill_value=1, name=df.name),
-                build_series(df, size=2, fill_value=2, name=df.name),
-            ])
-            if df.dtype == np.dtype('O'):
-                test_obj = test_obj.radd('O')
+            test_obj = build_series(df, size=[2, 2], fill_value=[1, 2], name=df.name, ensure_string=True)
 
         result_df = test_obj.agg(self.raw_func, axis=self.axis, **self.raw_func_kw)
 
@@ -285,6 +232,7 @@ class DataFrameAggregate(DataFrameOperand, DataFrameOperandMixin):
             agg_chunks_shape = (len(func_infos), in_df.chunk_shape[1])
 
         agg_chunks = np.empty(agg_chunks_shape, dtype=np.object)
+        dtypes_cache = dict()
         for chunk in in_df.chunks:
             input_index = chunk.index[1 - axis] if len(chunk.index) > 1 else 0
             if input_index not in input_index_to_output:
@@ -312,12 +260,22 @@ class DataFrameAggregate(DataFrameOperand, DataFrameOperandMixin):
                     else:
                         columns_value = out_df.index_value
                         index_value = parse_index(pd.Index([0]), out_df.key)
+
+                    try:
+                        dtypes = dtypes_cache[chunk.index[1]]
+                    except KeyError:
+                        dtypes = chunk.dtypes.reindex(columns_value.to_pandas()).dropna()
+                        dtypes_cache[chunk.index[1]] = dtypes
+
+                    agg_chunk = map_op.new_chunk([chunk], shape=shape, index=new_index, dtypes=dtypes,
+                                                 columns_value=columns_value, index_value=index_value)
                 else:
                     shape = (out_df.shape[0], 1)
                     columns_value = parse_index(pd.Index([0]), out_df.key, store_data=True)
                     index_value = out_df.index_value
-                agg_chunk = map_op.new_chunk([chunk], shape=shape, index=new_index,
-                                             columns_value=columns_value, index_value=index_value)
+
+                    agg_chunk = map_op.new_chunk([chunk], shape=shape, index=new_index,
+                                                 columns_value=columns_value, index_value=index_value)
             else:
                 agg_chunk = map_op.new_chunk([chunk], shape=(1,), index=new_index)
             agg_chunks[agg_chunk.index] = agg_chunk
@@ -555,7 +513,7 @@ class DataFrameAggregate(DataFrameOperand, DataFrameOperandMixin):
 
         if len(in_df.chunks) == 1:
             return cls._tile_single_chunk(op)
-        elif in_df.ndim == 2 and op.raw_func == 'size':
+        elif not _agg_size_as_series and in_df.ndim == 2 and op.raw_func == 'size':
             return cls._tile_size(op)
         else:
             return cls._tile_tree(op)
@@ -567,14 +525,20 @@ class DataFrameAggregate(DataFrameOperand, DataFrameOperandMixin):
         ndim = op.inputs[0].ndim
 
         if ndim == 2:
+            dtype = None
             if isinstance(value, (np.generic, int, float, complex)):
                 value = xdf.DataFrame([value], columns=index)
             elif not isinstance(value, xdf.DataFrame):
                 new_index = None if not op.gpu else getattr(value, 'index', None)
+                dtype = getattr(value, 'dtype', None)
                 value = xdf.DataFrame(value, columns=index, index=new_index)
             else:
                 return value
-            return value.T if axis == 0 else value
+
+            value = value.T if axis == 0 else value
+            if dtype == np.dtype('O') and getattr(op.outputs[0], 'dtypes', None) is not None:
+                value = value.astype(op.outputs[0].dtypes)
+            return value
         else:
             if isinstance(value, (np.generic, int, float, complex)):
                 value = xdf.Series([value], index=index)
@@ -761,13 +725,18 @@ class DataFrameAggregate(DataFrameOperand, DataFrameOperandMixin):
                 cls._execute_combine(ctx, op)
             elif op.stage == OperandStage.agg:
                 cls._execute_agg(ctx, op)
-            elif op.raw_func == 'size':
+            elif not _agg_size_as_series and op.raw_func == 'size':
                 xp = cp if op.gpu else np
                 ctx[op.outputs[0].key] = xp.array(ctx[op.inputs[0].key].agg(op.raw_func, axis=op.axis)) \
                     .reshape(op.outputs[0].shape)
             else:
                 xp = cp if op.gpu else np
-                result = ctx[op.inputs[0].key].agg(op.raw_func, axis=op.axis)
+                in_obj = op.inputs[0]
+                if isinstance(in_obj, INDEX_CHUNK_TYPE):
+                    result = op.func[0](ctx[in_obj.key])
+                else:
+                    result = ctx[in_obj.key].agg(op.raw_func, axis=op.axis)
+
                 if op.output_types[0] == OutputType.tensor:
                     result = xp.array(result)
                 ctx[op.outputs[0].key] = result
