@@ -14,14 +14,14 @@
 
 import functools
 from collections import OrderedDict
-from typing import NamedTuple, Any, List, Dict, Union, Callable
+from typing import Any, Callable, Dict, List, NamedTuple, Optional
 
 import numpy as np
 import pandas as pd
 
 from ...core import OutputType, Entity, Base
 from ...operands import OperandStage
-from ...utils import tokenize, is_build_mode, enter_mode, recursive_tile
+from ...utils import tokenize, is_build_mode, is_kernel_mode, enter_mode, recursive_tile
 from ...serialize import BoolField, AnyField, DataTypeField, Int32Field
 from ..core import SERIES_TYPE
 from ..utils import parse_index, build_df, build_empty_df, build_series, \
@@ -85,6 +85,10 @@ class DataFrameReductionOperand(DataFrameOperand):
     def use_inf_as_na(self):
         return self._use_inf_as_na
 
+    @property
+    def is_atomic(self):
+        return False
+
     def get_reduction_args(self, axis=None):
         args = dict(skipna=self.skipna)
         if self.inputs[0].ndim > 1:
@@ -136,7 +140,7 @@ def _default_agg_fun(value, func_name=None, **kw):
 
 class DataFrameReductionMixin(DataFrameOperandMixin):
     @classmethod
-    def _make_agg_object(cls, op):
+    def get_reduction_callable(cls, op):
         func_name = getattr(op, '_func_name')
         kw = dict(skipna=op.skipna, numeric_only=op.numeric_only,
                   bool_only=op.bool_only)
@@ -162,14 +166,14 @@ class DataFrameReductionMixin(DataFrameOperandMixin):
             dtypes, index = out_df.dtype, None
 
         out_df = recursive_tile(in_df.agg(
-            cls._make_agg_object(op), axis=op.axis or 0, _numeric_only=op.numeric_only,
+            cls.get_reduction_callable(op), axis=op.axis or 0, _numeric_only=op.numeric_only,
             _bool_only=op.bool_only, _combine_size=op.combine_size, _output_type=output_type,
             _dtypes=dtypes, _index=index
         ))
         return [out_df]
 
     def _call_groupby_level(self, df, level):
-        return df.groupby(level=level).agg(self._make_agg_object(self))
+        return df.groupby(level=level).agg(self.get_reduction_callable(self))
 
     def _call_dataframe(self, df):
         axis = getattr(self, 'axis', None) or 0
@@ -183,7 +187,7 @@ class DataFrameReductionMixin(DataFrameOperandMixin):
         if level is not None and axis == 1:
             raise NotImplementedError('Not support specify level for axis==1')
 
-        empty_df = build_df(df)
+        empty_df = build_df(df, ensure_string=True)
         if func_name == 'count':
             reduced_df = getattr(empty_df, func_name)(axis=axis, level=level, numeric_only=numeric_only)
         elif func_name == 'nunique':
@@ -204,8 +208,10 @@ class DataFrameReductionMixin(DataFrameOperandMixin):
             return self._call_groupby_level(df[list(reduced_df.columns)], level)
 
         reduced_shape = (df.shape[0],) if axis == 1 else reduced_df.shape
+        index_value = parse_index(reduced_df.index, store_data=True) \
+            if axis == 0 else parse_index(pd.RangeIndex(-1))
         return self.new_series([df], shape=reduced_shape, dtype=reduced_df.dtype,
-                               index_value=parse_index(reduced_df.index, store_data=axis == 0))
+                               index_value=index_value)
 
     def _call_series(self, series):
         level = getattr(self, 'level', None)
@@ -219,7 +225,7 @@ class DataFrameReductionMixin(DataFrameOperandMixin):
         if level is not None:
             return self._call_groupby_level(series, level)
 
-        empty_series = build_series(series)
+        empty_series = build_series(series, ensure_string=True)
         if func_name == 'count':
             reduced_series = empty_series.count(level=level)
         elif func_name == 'nunique':
@@ -239,6 +245,9 @@ class DataFrameReductionMixin(DataFrameOperandMixin):
         return self.new_scalar([series], dtype=np.array(reduced_series).dtype)
 
     def __call__(self, a):
+        if is_kernel_mode() and not getattr(self, 'is_atomic', False):
+            return self.get_reduction_callable(self)(a)
+
         if isinstance(a, DATAFRAME_TYPE):
             return self._call_dataframe(a)
         else:
@@ -429,8 +438,8 @@ class DataFrameCumReductionMixin(DataFrameOperandMixin):
 
 
 class CustomReduction:
-    name: Union[str, None]
-    output_limit: Union[int, None]
+    name: Optional[str]
+    output_limit: Optional[int]
     kwds: Dict
 
     # set to True when pre() already performs aggregation
@@ -488,15 +497,15 @@ class CustomReduction:
 class ReductionPreStep(NamedTuple):
     input_key: str
     output_key: str
-    columns: Union[List[str], None]
+    columns: Optional[List[str]]
     func: Callable
 
 
 class ReductionAggStep(NamedTuple):
     input_key: str
-    map_func_name: Union[str, None]
-    agg_func_name: Union[str, None]
-    custom_reduction: Union[CustomReduction, None]
+    map_func_name: Optional[str]
+    agg_func_name: Optional[str]
+    custom_reduction: Optional[CustomReduction]
     output_key: str
     output_limit: int
     kwds: Dict[str, Any]
@@ -506,7 +515,7 @@ class ReductionPostStep(NamedTuple):
     input_keys: List[str]
     output_key: str
     func_name: str
-    columns: Union[List[str], None]
+    columns: Optional[List[str]]
     func: Callable
 
 
@@ -585,6 +594,17 @@ class ReductionCompiler:
                 raise ValueError(f'Variable {var_name} used by {func.__name__} '
                                  'cannot be a Mars object')
 
+    @staticmethod
+    def _update_col_dict(col_dict: Dict, key: str, cols: List):
+        if key in col_dict:
+            existing_cols = col_dict[key]
+            if existing_cols is not None:
+                existing_col_set = set(existing_cols)
+                col_dict[key].extend(
+                    [c for c in cols if c not in existing_col_set])
+        else:
+            col_dict[key] = list(cols) if cols is not None else None
+
     def add_function(self, func, ndim, cols=None, func_name=None):
         cols = cols if cols is not None and self._axis == 0 else None
 
@@ -601,21 +621,14 @@ class ReductionCompiler:
 
         for step in compile_result.pre_funcs:
             self._output_key_to_pre_steps[step.output_key] = step
-            if step.output_key in self._output_key_to_pre_cols:
-                existing_cols = self._output_key_to_pre_cols[step.output_key]
-                if existing_cols is not None:
-                    existing_col_set = set(existing_cols)
-                    self._output_key_to_pre_cols[step.output_key].extend(
-                        [c for c in cols if c not in existing_col_set])
-            else:
-                self._output_key_to_pre_cols[step.output_key] = list(cols) if cols is not None else None
+            self._update_col_dict(self._output_key_to_pre_cols, step.output_key, cols)
 
         for step in compile_result.agg_funcs:
             self._output_key_to_agg_steps[step.output_key] = step
 
         for step in compile_result.post_funcs:
             self._output_key_to_post_steps[step.output_key] = step
-            self._output_key_to_post_cols[step.output_key] = cols
+            self._update_col_dict(self._output_key_to_post_cols, step.output_key, cols)
 
     @functools.lru_cache(100)
     def _compile_expr_function(self, py_src):
@@ -641,13 +654,11 @@ class ReductionCompiler:
             mock_obj = MarsDataFrame(mock_df)
 
         # calc target tileable to generate DAG
-        return func(mock_obj)
+        with enter_mode(kernel=True):
+            return func(mock_obj)
 
     @enter_mode(build=True)
     def _compile_function(self, func, func_name=None, ndim=1) -> ReductionSteps:
-        from . import DataFrameAll, DataFrameAny, DataFrameSum, DataFrameProd, \
-            DataFrameCount, DataFrameMin, DataFrameMax, DataFrameSize, \
-            DataFrameStrConcat, DataFrameCustomReduction
         from ...tensor.arithmetic.core import TensorBinOp, TensorUnaryOp
         from ...tensor.base import TensorWhere
         from ..arithmetic.core import DataFrameBinOp, DataFrameUnaryOp
@@ -660,10 +671,6 @@ class ReductionCompiler:
             return _func_compile_cache[func_token]
         custom_reduction = func if isinstance(func, CustomReduction) else None
 
-        atomic_agg_op_types = (DataFrameAll, DataFrameAny, DataFrameSum, DataFrameProd,
-                               DataFrameCount, DataFrameMin, DataFrameMax, DataFrameSize,
-                               DataFrameStrConcat, DataFrameCustomReduction)
-
         self._check_function_valid(func)
 
         try:
@@ -679,7 +686,7 @@ class ReductionCompiler:
             raise ValueError('Function not a reduction')
 
         agg_graph = func_ret.build_graph()
-        agg_tileables = set(t for t in agg_graph if isinstance(t.op, atomic_agg_op_types))
+        agg_tileables = set(t for t in agg_graph if getattr(t.op, 'is_atomic', False))
         # check operands before aggregation
         for t in agg_graph.dfs(list(agg_tileables), visit_predicate='all', reverse=True):
             if t not in agg_tileables and \
@@ -800,6 +807,10 @@ class ReductionCompiler:
                 func_name = func_name_raw = getattr(t.op, '_func_name', None)
                 rfunc_name = getattr(t.op, '_rfunc_name', func_name)
 
+                if func_name is None:
+                    func_name = func_name_raw = getattr(t.op, '_bit_func_name', None)
+                    rfunc_name = getattr(t.op, '_bit_rfunc_name', func_name)
+
                 # handle function name differences between numpy and pandas arithmetic ops
                 if func_name in _func_name_converts:
                     func_name = _func_name_converts[func_name]
