@@ -15,8 +15,6 @@
 import functools
 import itertools
 import operator
-import sys
-import threading
 from contextlib import contextmanager
 from numbers import Integral
 
@@ -25,6 +23,7 @@ import pandas as pd
 from pandas.api.types import is_string_dtype
 from pandas.api.extensions import ExtensionDtype
 from pandas.core.dtypes.cast import find_common_type
+
 try:
     import pyarrow as pa
 except ImportError:  # pragma: no cover
@@ -66,8 +65,8 @@ def hash_dataframe_on(df, on, size, level=None):
         else:
             data = df[on]
         hashed_label = pd.util.hash_pandas_object(data, index=False, categorize=False)
-    idx_to_grouped = df.index.groupby(hashed_label % size)
-    return [idx_to_grouped.get(i, pd.Index([])).unique() for i in range(size)]
+    idx_to_grouped = pd.RangeIndex(0, len(hashed_label)).groupby(hashed_label % size)
+    return [idx_to_grouped.get(i, pd.Index([])) for i in range(size)]
 
 
 def hash_dtypes(dtypes, size):
@@ -171,8 +170,11 @@ def decide_dataframe_chunk_sizes(shape, chunk_size, memory_usage):
             col_chunk_store.append(average_memory_usage.iloc[start: start + cs].sum())
             col_left_size -= cs
         if row_left_size > 0:
-            max_col_chunk_store = max(col_chunk_store)
-            cs = min(row_left_size, int(max_chunk_size / max_col_chunk_store))
+            if col_chunk_store:
+                max_col_chunk_store = max(col_chunk_store)
+                cs = min(row_left_size, int(max_chunk_size / max_col_chunk_store))
+            else:
+                cs = row_left_size
             row_chunk_size.append(cs)
             row_left_size -= cs
 
@@ -230,7 +232,7 @@ def parse_index(index_value, *args, store_data=False, key=None):
     def _get_index_min(index):
         try:
             return index.min()
-        except ValueError:
+        except (ValueError, AttributeError):
             if isinstance(index, pd.IntervalIndex):
                 return None
             raise
@@ -240,7 +242,7 @@ def parse_index(index_value, *args, store_data=False, key=None):
     def _get_index_max(index):
         try:
             return index.max()
-        except ValueError:
+        except (ValueError, AttributeError):
             if isinstance(index, pd.IntervalIndex):
                 return None
             raise
@@ -469,38 +471,64 @@ def _generate_value(dtype, fill_value):
 
 def build_empty_df(dtypes, index=None):
     columns = dtypes.index
+    length = len(index) if index is not None else 0
+    record = [[_generate_value(dtype, 1) for dtype in dtypes]] * max(1, length)
+
     # duplicate column may exist,
     # so use RangeIndex first
-    df = pd.DataFrame(columns=pd.RangeIndex(len(columns)), index=index)
-    length = len(index) if index is not None else 0
-    for i, d in enumerate(dtypes):
-        df[i] = pd.Series([_generate_value(d, 1) for _ in range(length)],
-                          dtype=d, index=index)
-    df.columns = columns
-    return df
-
-
-def build_df(df_obj, fill_value=1, size=1):
-    empty_df = build_empty_df(df_obj.dtypes, index=df_obj.index_value.to_pandas()[:0])
-    dtypes = empty_df.dtypes
-    record = [_generate_value(dtype, fill_value) for dtype in dtypes]
-    if len(record) != 0:  # columns is empty in some cases
-        if isinstance(empty_df.index, pd.MultiIndex):
-            index = tuple(_generate_value(level.dtype, fill_value) for level in empty_df.index.levels)
-            empty_df = empty_df.reindex(
-                index=pd.MultiIndex.from_tuples([index], names=empty_df.index.names))
-            empty_df.iloc[0] = record
-        else:
-            index = _generate_value(empty_df.index.dtype, fill_value)
-            empty_df.loc[index] = record
-
-    empty_df = pd.concat([empty_df] * size)
-    # make sure dtypes correct for MultiIndex
-    for i, dtype in enumerate(dtypes.tolist()):
-        s = empty_df.iloc[:, i]
+    df = pd.DataFrame(record, columns=range(len(dtypes)), index=index)
+    for i, dtype in enumerate(dtypes):
+        s = df.iloc[:, i]
         if not pd.api.types.is_dtype_equal(s.dtype, dtype):
-            empty_df.iloc[:, i] = s.astype(dtype)
-    return empty_df
+            df.iloc[:, i] = s.astype(dtype)
+
+    df.columns = columns
+    return df[:length] if len(df) > length else df
+
+
+def build_df(df_obj, fill_value=1, size=1, ensure_string=False):
+    dfs = []
+    if not isinstance(size, (list, tuple)):
+        sizes = [size]
+    else:
+        sizes = size
+
+    if not isinstance(fill_value, (list, tuple)):
+        fill_values = [fill_value]
+    else:
+        fill_values = fill_value
+
+    for size, fill_value in zip(sizes, fill_values):
+        dtypes = df_obj.dtypes
+        record = [[_generate_value(dtype, fill_value) for dtype in dtypes]] * size
+        df = pd.DataFrame(record)
+        df.columns = dtypes.index
+
+        if len(record) != 0:  # columns is empty in some cases
+            target_index = df_obj.index_value.to_pandas()
+            if isinstance(target_index, pd.MultiIndex):
+                index_val = tuple(_generate_value(level.dtype, fill_value)
+                                  for level in target_index.levels)
+                df.index = pd.MultiIndex.from_tuples([index_val] * size, names=target_index.names)
+            else:
+                index_val = _generate_value(target_index.dtype, fill_value)
+                df.index = pd.Index([index_val] * size, name=target_index.name)
+
+        # make sure dtypes correct
+        for i, dtype in enumerate(dtypes):
+            s = df.iloc[:, i]
+            if not pd.api.types.is_dtype_equal(s.dtype, dtype):
+                df.iloc[:, i] = s.astype(dtype)
+        dfs.append(df)
+    if len(dfs) == 1:
+        ret_df = dfs[0]
+    else:
+        ret_df = pd.concat(dfs)
+
+    if ensure_string:
+        obj_dtypes = df_obj.dtypes[df_obj.dtypes == np.dtype('O')]
+        ret_df[obj_dtypes.index] = ret_df[obj_dtypes.index].radd('O')
+    return ret_df
 
 
 def build_empty_series(dtype, index=None, name=None):
@@ -509,33 +537,54 @@ def build_empty_series(dtype, index=None, name=None):
                      dtype=dtype, index=index, name=name)
 
 
-def build_series(series_obj, fill_value=1, size=1, name=None):
-    empty_series = build_empty_series(series_obj.dtype, name=name,
-                                      index=series_obj.index_value.to_pandas()[:0])
-    record = _generate_value(series_obj.dtype, fill_value)
-    if isinstance(empty_series.index, pd.MultiIndex):
-        index = tuple(_generate_value(level.dtype, fill_value) for level in empty_series.index.levels)
-        empty_series = empty_series.reindex(
-            index=pd.MultiIndex.from_tuples([index], names=empty_series.index.names))
-        empty_series.iloc[0] = record
+def build_series(series_obj, fill_value=1, size=1, name=None, ensure_string=False):
+    seriess = []
+    if not isinstance(size, (list, tuple)):
+        sizes = [size]
     else:
-        if isinstance(empty_series.index.dtype, pd.CategoricalDtype):
-            index = None
-        else:
-            index = _generate_value(empty_series.index.dtype, fill_value)
-        empty_series.loc[index] = record
+        sizes = size
 
-    empty_series = pd.concat([empty_series] * size)
-    # make sure dtype correct for MultiIndex
-    empty_series = empty_series.astype(series_obj.dtype, copy=False)
-    return empty_series
+    if not isinstance(fill_value, (list, tuple)):
+        fill_values = [fill_value]
+    else:
+        fill_values = fill_value
+
+    for size, fill_value in zip(sizes, fill_values):
+        empty_series = build_empty_series(series_obj.dtype, name=name,
+                                          index=series_obj.index_value.to_pandas()[:0])
+        record = _generate_value(series_obj.dtype, fill_value)
+        if isinstance(empty_series.index, pd.MultiIndex):
+            index = tuple(_generate_value(level.dtype, fill_value) for level in empty_series.index.levels)
+            empty_series = empty_series.reindex(
+                index=pd.MultiIndex.from_tuples([index], names=empty_series.index.names))
+            empty_series.iloc[0] = record
+        else:
+            if isinstance(empty_series.index.dtype, pd.CategoricalDtype):
+                index = None
+            else:
+                index = _generate_value(empty_series.index.dtype, fill_value)
+            empty_series.loc[index] = record
+
+        empty_series = pd.concat([empty_series] * size)
+        # make sure dtype correct for MultiIndex
+        empty_series = empty_series.astype(series_obj.dtype, copy=False)
+        seriess.append(empty_series)
+
+    if len(seriess) == 1:
+        ret_series = seriess[0]
+    else:
+        ret_series = pd.concat(seriess)
+
+    if ensure_string and series_obj.dtype == np.dtype('O'):
+        ret_series = ret_series.radd('O')
+    return ret_series
 
 
 def concat_index_value(index_values, store_data=False):
-    result = pd.Index([])
     if not isinstance(index_values, (list, tuple)):
         index_values = [index_values]
-    for index_value in index_values:
+    result = index_values[0].value.to_pandas()
+    for index_value in index_values[1:]:
         if isinstance(index_value, pd.Index):
             result = result.append(index_value)
         else:
@@ -666,40 +715,65 @@ def indexing_index_value(index_value, indexes, store_data=False):
             return parse_index(pd_index[indexes], store_data=store_data)
 
 
-def merge_index_value(to_merge_index_values, store_data=False):
+def merge_index_value(to_merge_index_values: dict, store_data: bool = False):
     """
     Merge index value according to their chunk index.
-    :param to_merge_index_values: Dict object. {index: index_value}
-    :return: Merged index_value
+
+    Parameters
+    ----------
+    to_merge_index_values : dict
+        index to index_value
+    store_data : bool
+        store data in index_value
+
+    Returns
+    -------
+    merged_index_value
     """
-    index_value = None
+
+    pd_index = None
     min_val, min_val_close, max_val, max_val_close = None, None, None, None
     for _, chunk_index_value in sorted(to_merge_index_values.items()):
-        if index_value is None:
-            index_value = chunk_index_value.to_pandas()
+        if pd_index is None:
+            pd_index = chunk_index_value.to_pandas()
             min_val, min_val_close, max_val, max_val_close = \
                 chunk_index_value.min_val, \
                 chunk_index_value.min_val_close, \
                 chunk_index_value.max_val, \
                 chunk_index_value.max_val_close
         else:
-            index_value = index_value.append(chunk_index_value.to_pandas())
+            cur_pd_index = chunk_index_value.to_pandas()
+            if store_data or (
+                    isinstance(pd_index, pd.RangeIndex) and
+                    isinstance(cur_pd_index, pd.RangeIndex) and
+                    cur_pd_index.step == pd_index.step and
+                    cur_pd_index.start == pd_index.stop
+            ):
+                # range index that is continuous
+                pd_index = pd_index.append(cur_pd_index)
+            else:
+                pd_index = pd.Index([], dtype=pd_index.dtype)
             if chunk_index_value.min_val is not None:
-                if min_val is None or min_val > chunk_index_value.min_val:
-                    min_val = chunk_index_value.min_val
-                    min_val_close = chunk_index_value.min_val_close
+                try:
+                    if min_val is None or min_val > chunk_index_value.min_val:
+                        min_val = chunk_index_value.min_val
+                        min_val_close = chunk_index_value.min_val_close
+                except TypeError:
+                    # min_value has different types that cannot compare
+                    # just stop compare
+                    continue
             if chunk_index_value.max_val is not None:
                 if max_val is None or max_val < chunk_index_value.max_val:
                     max_val = chunk_index_value.max_val
                     max_val_close = chunk_index_value.max_val_close
 
-    new_index_value = parse_index(index_value, store_data=store_data)
-    if not new_index_value.has_value():
-        new_index_value._index_value._min_val = min_val
-        new_index_value._index_value._min_val_close = min_val_close
-        new_index_value._index_value._max_val = max_val
-        new_index_value._index_value._max_val_close = max_val_close
-    return new_index_value
+    index_value = parse_index(pd_index, store_data=store_data)
+    if not index_value.has_value():
+        index_value._index_value._min_val = min_val
+        index_value._index_value._min_val_close = min_val_close
+        index_value._index_value._max_val = max_val
+        index_value._index_value._max_val_close = max_val_close
+    return index_value
 
 
 def infer_dtypes(left_dtypes, right_dtypes, operator):
@@ -896,7 +970,7 @@ def fetch_corner_data(df_or_series, session=None) -> pd.DataFrame:
         index_size = min_rows // 2 + 1
 
     if index_size is None:
-        return df_or_series.fetch(session=session)
+        return df_or_series._fetch(session=session)
     else:
         head = iloc(df_or_series)[:index_size]
         tail = iloc(df_or_series)[-index_size:]
@@ -1041,38 +1115,3 @@ def make_dtypes(dtypes):
         else:
             dtypes = pd.Series(dtypes)
     return dtypes.apply(make_dtype)
-
-
-_io_quiet_local = threading.local()
-_io_quiet_lock = threading.Lock()
-
-
-class _QuietIOWrapper:
-    def __init__(self, wrapped):
-        self.wrapped = wrapped
-
-    def __getattr__(self, item):
-        return getattr(self.wrapped, item)
-
-    def write(self, d):
-        if getattr(_io_quiet_local, 'is_wrapped', False):
-            return 0
-        return self.wrapped.write(d)
-
-
-@contextmanager
-def quiet_stdio():
-    """Quiets standard outputs when inferring types of functions"""
-    with _io_quiet_lock:
-        _io_quiet_local.is_wrapped = True
-        sys.stdout = _QuietIOWrapper(sys.stdout)
-        sys.stderr = _QuietIOWrapper(sys.stderr)
-
-    try:
-        yield
-    finally:
-        with _io_quiet_lock:
-            sys.stdout = sys.stdout.wrapped
-            sys.stderr = sys.stderr.wrapped
-            if not isinstance(sys.stdout, _QuietIOWrapper):
-                _io_quiet_local.is_wrapped = False
