@@ -26,7 +26,8 @@ from .core import Entity, Chunk, Tileable, AttributeAsDictKey, ExecutableTuple,
     FuseChunkData, FuseChunk, OutputType, get_chunk_types, get_tileable_types, \
     register_fetch_class, get_fetch_class, get_output_types
 from .serialize import SerializableMetaclass, ValueType, ProviderType, IdentityField, \
-    ListField, DictField, Int32Field, BoolField, StringField, ReferenceField
+    ListField, DictField, Int32Field, Float32Field, BoolField, StringField, \
+    ReferenceField
 from .tiles import NotSupportTile
 from .utils import AttributeDict, to_str, calc_data_size, calc_object_overhead, \
     enter_mode, is_eager_mode
@@ -35,6 +36,7 @@ from .utils import AttributeDict, to_str, calc_data_size, calc_object_overhead,
 operand_type_to_oprand_cls = {}
 OP_TYPE_KEY = '_op_type_'
 OP_MODULE_KEY = '_op_module_'
+OP_REGISTER_CALLBACK = '_on_op_register_'
 T = TypeVar('T')
 
 
@@ -52,11 +54,17 @@ class OperandMetaclass(SerializableMetaclass):
                 kv[OP_TYPE_KEY] = getattr(base, OP_TYPE_KEY)
             if OP_MODULE_KEY not in kv and hasattr(base, OP_MODULE_KEY):
                 kv[OP_MODULE_KEY] = getattr(base, OP_MODULE_KEY)
+            if OP_REGISTER_CALLBACK not in kv and hasattr(base, OP_REGISTER_CALLBACK):
+                kv[OP_REGISTER_CALLBACK] = getattr(base, OP_REGISTER_CALLBACK)
 
         if kv.get(OP_TYPE_KEY) is not None and kv.get(OP_MODULE_KEY) is not None:
             # common operand can be inherited for different modules, like tensor or dataframe, so forth
             operand_type_to_oprand_cls[kv[OP_MODULE_KEY], kv[OP_TYPE_KEY]] = cls
 
+        reg_callback = getattr(cls, OP_REGISTER_CALLBACK, None)
+        if callable(reg_callback):
+            reg_callback()
+
         return cls
 
 
@@ -96,7 +104,7 @@ class Operand(AttributeAsDictKey, metaclass=OperandMetaclass):
                               on_serialize=OutputType.serialize_list,
                               on_deserialize=OutputType.deserialize_list)
 
-    _memory_scale = Int32Field('memory_scale')
+    _memory_scale = Float32Field('memory_scale')
 
     _stage = Int32Field('stage', on_serialize=lambda s: s.value if s is not None else s,
                         on_deserialize=lambda n: OperandStage(n) if n is not None else n)
@@ -196,7 +204,7 @@ class Operand(AttributeAsDictKey, metaclass=OperandMetaclass):
 
     @property
     def memory_scale(self):
-        return getattr(self, '_memory_scale', None) or 1
+        return getattr(self, '_memory_scale', None)
 
     @property
     def stage(self) -> Union[None, "OperandStage"]:
@@ -510,6 +518,7 @@ class TileableOperandMixin(object):
                 pass
 
         exec_size = max(exec_size, total_out_size)
+        memory_scale = op.memory_scale or 1.0
         for out in outputs:
             if out.key in ctx:
                 continue
@@ -527,7 +536,7 @@ class TileableOperandMixin(object):
                 max_sparse_size = np.nan
             if not np.isnan(max_sparse_size):
                 store_size = min(store_size, max_sparse_size)
-            ctx[out.key] = (store_size, exec_size * op.memory_scale // len(outputs))
+            ctx[out.key] = (store_size, exec_size * memory_scale // len(outputs))
 
     @classmethod
     def concat_tileable_chunks(cls, tileable):
@@ -583,8 +592,10 @@ class MapReduceOperand(Operand):
             inputs = self.inputs or ()
             deps = []
             for inp in inputs:
-                if isinstance(inp.op, (ShuffleProxy, FetchShuffle)):
+                if isinstance(inp.op, ShuffleProxy):
                     deps.extend([(chunk.key, self._shuffle_key) for chunk in inp.inputs or ()])
+                elif isinstance(inp.op, FetchShuffle):
+                    deps.extend([(k, self._shuffle_key) for k in inp.op.to_fetch_keys])
                 else:
                     deps.append(inp.key)
             return deps
@@ -594,6 +605,15 @@ class MapReduceOperand(Operand):
 class ShuffleProxy(VirtualOperand):
     _op_type_ = OperandDef.SHUFFLE_PROXY
 
+    _assign_reducers = BoolField('assign_reducers')
+
+    def __init__(self, assign_reducers=True, **kw):
+        super().__init__(_assign_reducers=assign_reducers, **kw)
+
+    @property
+    def assign_reducers(self) -> bool:
+        return self._assign_reducers
+
 
 class Fetch(Operand):
     _op_type_ = OperandDef.FETCH
