@@ -26,7 +26,7 @@ from .operands import OperandState
 logger = logging.getLogger(__name__)
 
 
-class GraphAnalyzer(object):
+class GraphAnalyzer:
     """
     Analyzer for chunk graph, supporting optimization analysis
     as well as fail-over analysis.
@@ -359,9 +359,12 @@ class GraphAnalyzer(object):
         sorted_candidates = [v for v in chunks_to_assign]
         while max(worker_quotas.values()):
             worker = max(worker_quotas, key=lambda k: worker_quotas[k])
-            cur = sorted_candidates.pop()
-            while cur.op.key in cur_assigns:
+            try:
                 cur = sorted_candidates.pop()
+                while cur.op.key in cur_assigns:
+                    cur = sorted_candidates.pop()
+            except IndexError:  # pragma: no cover
+                break
             self._assign_by_bfs(cur, worker, worker_quotas, spread_ranges, op_keys,
                                 cur_assigns, graph=graph)
 
