@@ -37,6 +37,7 @@ from ..config import options
 from ..errors import ExecutionInterrupted, GraphNotExists, WorkerDead
 from ..graph import DAG
 from ..operands import Fetch, ShuffleProxy, VirtualOperand, SuccessorsExclusive
+from ..optimizes.tileable_graph import OptimizeIntegratedTileableGraphBuilder
 from ..serialize import dataserializer
 from ..tiles import handler, IterativeChunkGraphBuilder, \
     TileableGraphBuilder, get_tiled
@@ -592,6 +593,13 @@ class GraphActor(SchedulerActor):
 
         logger.debug(f'Terminal chunk keys: {self._terminal_chunk_keys}')
 
+    @staticmethod
+    def _get_tileable_graph_builder(**kwargs):
+        if options.optimize_tileable_graph:
+            return OptimizeIntegratedTileableGraphBuilder(**kwargs)
+        else:
+            return TileableGraphBuilder(**kwargs)
+
     @log_unhandled
     @enter_mode(build=True)
     def prepare_graph(self, compose=True):
@@ -607,6 +615,16 @@ class GraphActor(SchedulerActor):
             # gen target tileable keys if not provided
             self._scan_tileable_graph()
 
+            if options.optimize_tileable_graph:
+                target_tileables = [n for n in tileable_graph
+                                    if n.key in self._target_tileable_keys]
+                optimized_graph_builder = OptimizeIntegratedTileableGraphBuilder()
+                self._tileable_graph_cache = tileable_graph = \
+                    optimized_graph_builder.build(target_tileables)
+                # rescan
+                self._target_tileable_datas = list()
+                self._scan_tileable_graph()
+
             def on_tile(raw_tileables, tileds):
                 first = tileds[0]
                 if any(inp in self._tileable_to_fetch for inp in raw_tileables[0].inputs):
@@ -651,7 +669,7 @@ class GraphActor(SchedulerActor):
             # build tileable graph from failed ops and their inputs
             failed_tileable_set = set(itertools.chain(
                 *(op.outputs for op in chunk_graph_builder.interrupted_ops)))
-            tileable_graph_builder = TileableGraphBuilder(
+            tileable_graph_builder = self._get_tileable_graph_builder(
                 inputs_selector=lambda inps: [inp for inp in inps if inp in failed_tileable_set])
             to_run_tileable_graph = tileable_graph_builder.build(failed_tileable_set)
             to_fetch_tileables = []
@@ -836,12 +854,16 @@ class GraphActor(SchedulerActor):
         shared_input_chunk_keys = set()
         pure_dep_chunk_keys = set()
         chunk_keys = set()
+        virtual_chunk_keys = set()
         shuffle_keys = dict()
         predecessors_to_successors = dict()
+        assign_reducers = False
 
         for c in chunks:
             # handling predecessor args
             for pn in graph.iter_predecessors(c):
+                if isinstance(pn.op, VirtualOperand):
+                    virtual_chunk_keys.add(pn.key)
                 if not isinstance(pn.op, Fetch):
                     predecessor_keys.add(pn.op.key)
                 input_chunk_keys.add(pn.key)
@@ -864,6 +886,10 @@ class GraphActor(SchedulerActor):
 
             chunk_keys.update(co.key for co in c.op.outputs)
 
+            if isinstance(c.op, ShuffleProxy):
+                assign_reducers = c.op.assign_reducers \
+                    if c.op.assign_reducers is not None else True
+
         io_meta = dict(
             predecessors=list(predecessor_keys),
             successors=list(successor_keys),
@@ -872,6 +898,10 @@ class GraphActor(SchedulerActor):
             shared_input_chunks=list(shared_input_chunk_keys),
             chunks=list(chunk_keys),
         )
+        if virtual_chunk_keys:
+            io_meta['virtual_chunk_keys'] = list(virtual_chunk_keys)
+        if assign_reducers:
+            io_meta['assign_reducers'] = True
         if shuffle_keys:
             io_meta['shuffle_keys'] = [shuffle_keys.get(k) for k in io_meta['successors']]
         if predecessors_to_successors:
@@ -1197,12 +1227,12 @@ class GraphActor(SchedulerActor):
         :param filter_fields: filter the fields('nsplits', 'chunk_keys', 'chunk_indexes') in meta.
         :return: metas list.
         """
-        meta_names = ['nsplits', 'chunk_keys', 'chunk_indexes']
+        meta_names = ['nsplits', 'chunk_keys', 'chunk_indexes', 'extra_meta']
         filter_fields = filter_fields or meta_names
         for field in filter_fields:
-            if field not in ['nsplits', 'chunk_keys', 'chunk_indexes']:
+            if field not in ['nsplits', 'chunk_keys', 'chunk_indexes', 'extra_meta']:
                 raise ValueError(f'Field {field} is invalid')
-        ret_nsplits = 'nsplits' in filter_fields
+
         metas = []
         for tileable_key in tileable_keys:
             meta = dict()
@@ -1210,15 +1240,19 @@ class GraphActor(SchedulerActor):
             chunk_keys, chunk_indexes = tuple(zip(*[(c.key, c.index) for c in tileable.chunks]))
             meta['chunk_keys'] = chunk_keys
             meta['chunk_indexes'] = chunk_indexes
-            if ret_nsplits:
-                if hasattr(tileable, 'shape') and np.nan in tileable.shape:
-                    chunk_shapes = self.chunk_meta.batch_get_chunk_shape(self._session_id,  chunk_keys)
-                    nsplits = calc_nsplits(OrderedDict(zip(chunk_indexes, chunk_shapes)))
-                else:
-                    nsplits = tileable.nsplits
-                meta['nsplits'] = nsplits
-                if filter_fields is not None:
-                    metas.append([meta[k] for k in filter_fields])
+
+            if hasattr(tileable, 'shape') and np.nan in tileable.shape:
+                chunk_shapes = self.chunk_meta.batch_get_chunk_shape(self._session_id,  chunk_keys)
+                nsplits = calc_nsplits(OrderedDict(zip(chunk_indexes, chunk_shapes)))
+            else:
+                nsplits = tileable.nsplits
+            meta['nsplits'] = nsplits
+            extra_meta = tileable.params
+            extra_meta.pop('shape', None)
+            extra_meta = dict((k, v.to_pandas() if hasattr(v, 'to_pandas') else v)
+                              for k, v in extra_meta.items())
+            meta['extra_meta'] = extra_meta
+            metas.append([meta[k] for k in filter_fields])
         return metas
 
     def get_tileable_chunk_metas(self, tileable_keys, filter_fields=None):
