@@ -259,18 +259,7 @@ def loads(buf):
         data = decompressors[compress](mv[HEADER_LENGTH:])
 
     if header.type == SerialType.ARROW:
-        try:
-            return deserialize(memoryview(data))
-        except pyarrow.lib.ArrowInvalid:  # pragma: no cover
-            # reconstruct value from buffers of arrow components
-            data_view = memoryview(data)
-            meta_block_size = np.frombuffer(data_view[0:4], dtype='int32').item()
-            meta = pickle.loads(data_view[4:4 + meta_block_size])  # nosec
-            buffer_sizes = meta.pop('buffer_sizes')
-            bounds = np.cumsum([4 + meta_block_size] + buffer_sizes)
-            meta['data'] = [pyarrow.py_buffer(data_view[bounds[idx]:bounds[idx + 1]])
-                            for idx in range(len(buffer_sizes))]
-            return _patch_pandas_mgr(pyarrow.deserialize_components(meta, mars_serialize_context()))
+        return deserialize(memoryview(data))
     else:
         return _patch_pandas_mgr(pickle.loads(data))  # nosec
 
@@ -311,6 +300,7 @@ def _wrap_deprecates(fun):
     def _wrapped(*args, **kwargs):
         with warnings.catch_warnings():
             warnings.filterwarnings('ignore', category=DeprecationWarning)
+            warnings.filterwarnings('ignore', category=FutureWarning)
             return fun(*args, **kwargs)
 
     if pyarrow is not None and LooseVersion(pyarrow.__version__) >= '2.0':
@@ -592,7 +582,5 @@ def mars_serialize_context():
                           custom_serializer=_serialize_sparse_dtype,
                           custom_deserializer=_deserialize_sparse_dtype)
         _apply_pyarrow_serialization_patch(ctx)
-        if vineyard is not None:  # pragma: no cover
-            vineyard.register_vineyard_serialize_context(ctx)
         _serialize_context = ctx
     return _serialize_context
