@@ -264,7 +264,10 @@ class TensorReductionMixin(TensorOperandMixin):
         reduce_func = getattr(xp, func_name)
         out = op.outputs[0]
         with device(device_id):
-            if "dtype" in inspect.getfullargspec(reduce_func).args:
+            if input_chunk.size == 0 and op.keepdims:
+                # input chunk is empty, when keepdims is True, return itself
+                ret = input_chunk
+            elif "dtype" in inspect.getfullargspec(reduce_func).args:
                 ret = reduce_func(input_chunk, axis=axis,
                                   dtype=op.dtype,
                                   keepdims=bool(op.keepdims))
@@ -492,7 +495,7 @@ class TensorCumReductionMixin(TensorReductionMixin):
                 to_cum_chunks.append(sliced_chunk)
             to_cum_chunks.append(chunk)
 
-            bin_op = bin_op_type(dtype=chunk.dtype)
+            bin_op = bin_op_type(args=to_cum_chunks, dtype=chunk.dtype)
             output_chunk = bin_op.new_chunk(to_cum_chunks, shape=chunk.shape,
                                             index=chunk.index, order=out_tensor.order)
             output_chunks.append(output_chunk)
