@@ -34,7 +34,7 @@ class TensorPdist(TensorMapReduceOperand, TensorOperandMixin):
 
     _input = KeyField('input')
     _metric = AnyField('metric')
-    _p = Float16Field('p')
+    _p = Float16Field('p', on_serialize=lambda x: float(x) if x is not None else x)
     _w = KeyField('w')
     _v = KeyField('V')
     _vi = KeyField('VI')
