@@ -26,7 +26,7 @@ from ...utils import check_chunks_unknown_shape, recursive_tile
 from ..core import TENSOR_TYPE, TENSOR_CHUNK_TYPE, TensorOrder
 from ..operands import TensorOperand, TensorOperandMixin
 from ..datasource import tensor as astensor
-from ..arithmetic.utils import tree_add
+from ..arithmetic.utils import chunk_tree_add
 from ..utils import is_asc_sorted
 from ..array_utils import as_same_device, device
 
@@ -448,7 +448,7 @@ class TensorHistogramBinEdges(TensorOperand, TensorOperandMixin):
         super()._set_inputs(inputs)
         inputs_iter = iter(self._inputs)
         self._input = next(inputs_iter)
-        if isinstance(self._bins, TENSOR_TYPE):
+        if isinstance(self._bins, (TENSOR_TYPE, TENSOR_CHUNK_TYPE)):
             self._bins = next(inputs_iter)
         if self._weights is not None:
             self._weights = next(inputs_iter)
@@ -503,13 +503,13 @@ class TensorHistogramBinEdges(TensorOperand, TensorOperandMixin):
             inputs.append(bins)
         if weights is not None:
             inputs.append(weights)
-        if a.size > 0 and \
+        if (a.size > 0 or np.isnan(a.size)) and \
                 (isinstance(bins, str) or mt.ndim(bins) == 0) and not range:
-            # for bins that is str or integer,
-            # requires min max calculated first
-            input_min = self._input_min = a.min()
+            # for bins that is str or integer, requires min max calculated first
+            # dims need to be kept in case a is empty which causes errors in reduction
+            input_min = self._input_min = a.min(keepdims=True)
             inputs.append(input_min)
-            input_max = self._input_max = a.max()
+            input_max = self._input_max = a.max(keepdims=True)
             inputs.append(input_max)
 
         return self.new_tensor(inputs, shape=shape, order=TensorOrder.C_ORDER)
@@ -527,7 +527,10 @@ class TensorHistogramBinEdges(TensorOperand, TensorOperandMixin):
             metas = ctx.get_chunk_metas(min_max_chunk_keys)
             if any(meta is None for meta in metas):
                 raise TilesError('`input_min` or `input_max` need be executed first')
-            range_ = tuple(ctx.get_chunk_results(min_max_chunk_keys))
+            range_results = ctx.get_chunk_results(min_max_chunk_keys)
+            # make sure returned bounds are valid
+            if all(x.size > 0 for x in range_results):
+                range_ = tuple(x[0] for x in range_results)
         if isinstance(op.bins, TENSOR_TYPE):
             # `bins` is a Tensor, needs to be calculated first
             bins_chunk_keys = [c.key for c in op.bins.chunks]
@@ -801,7 +804,7 @@ class TensorHistogram(TensorOperand, TensorOperandMixin):
         super()._set_inputs(inputs)
         inputs_iter = iter(self._inputs)
         self._input = next(inputs_iter)
-        if isinstance(self._bins, TENSOR_TYPE):
+        if isinstance(self._bins, (TENSOR_TYPE, TENSOR_CHUNK_TYPE)):
             self._bins = next(inputs_iter)
         if self._weights is not None:
             self._weights = next(inputs_iter)
@@ -853,7 +856,7 @@ class TensorHistogram(TensorOperand, TensorOperandMixin):
             out_chunks.append(out_chunk)
 
         # merge chunks together
-        chunk = tree_add(out.dtype, out_chunks, (0,), shape)
+        chunk = chunk_tree_add(out.dtype, out_chunks, (0,), shape)
         new_op = op.copy()
         n = new_op.new_tensor(op.inputs, shape=shape, order=out.order,
                               chunks=[chunk], nsplits=((shape[0],),))
@@ -981,6 +984,7 @@ def histogram(a, bins=10, range=None, weights=None, density=None):
     >>> plt.show()
 
     """
+    a, weights = _ravel_and_check_weights(a, weights)
     op = TensorHistogram(input=a, bins=bins, range=range,
                          weights=weights, density=density)
     return op(a, bins, range, weights)
