@@ -33,7 +33,8 @@ import time
 import warnings
 import weakref
 import zlib
-from typing import List
+from contextlib import contextmanager
+from typing import List, Union
 
 import numpy as np
 import pandas as pd
@@ -197,30 +198,28 @@ def _get_ports_from_netstat():
     while True:
         p = subprocess.Popen('netstat -a -n -p tcp'.split(), stdout=subprocess.PIPE)
         try:
-            p.wait(5)
-            break
-        except subprocess.TimeoutExpired:
-            p.terminate()
-            continue
-    occupied = set()
-    for line in p.stdout:
-        line = to_str(line)
-        if '.' not in line:
-            continue
-        for part in line.split():
-            if '.' in part:
-                _, port_str = part.rsplit('.', 1)
-                if port_str == '*':
+            outs, _ = p.communicate(timeout=5)
+            outs = to_str(outs).split('\n')
+            occupied = set()
+            for line in outs:
+                if '.' not in line:
                     continue
-                port = int(port_str)
-                if LOW_PORT_BOUND <= port <= HIGH_PORT_BOUND:
-                    occupied.add(int(port_str))
-                break
-    p.stdout.close()
-    return occupied
+                for part in line.split():
+                    if '.' in part:
+                        _, port_str = part.rsplit('.', 1)
+                        if port_str == '*':
+                            continue
+                        port = int(port_str)
+                        if LOW_PORT_BOUND <= port <= HIGH_PORT_BOUND:
+                            occupied.add(int(port_str))
+                        break
+            return occupied
+        except subprocess.TimeoutExpired:
+            p.kill()
+            continue
 
 
-def get_next_port(typ=None):
+def get_next_port(typ=None, occupy=True):
     import psutil
     try:
         conns = psutil.net_connections()
@@ -237,7 +236,8 @@ def get_next_port(typ=None):
         if i in occupied:
             continue
         if idx == 0:
-            _local_occupied_ports.add(i)
+            if occupy:
+                _local_occupied_ports.add(i)
             return i
         idx -= 1
     raise SystemError('No ports available.')
@@ -278,9 +278,15 @@ def lazy_import(name, package=None, globals=None, locals=None, rename=None):
         return None
 
 
-def serialize_graph(graph, compress=False, data_serial_type=None, pickle_protocol=None):
-    ser_graph = graph.to_pb(data_serial_type=data_serial_type,
-                            pickle_protocol=pickle_protocol).SerializeToString()
+def serialize_graph(graph, compress=False, data_serial_type=None, pickle_protocol=None,
+                    serialize_method=None):
+    serialize_method = serialize_method or options.serialize_method
+    if serialize_method == 'json':
+        ser_graph = json.dumps(graph.to_json(data_serial_type=data_serial_type,
+                                             pickle_protocol=pickle_protocol))
+    else:
+        ser_graph = graph.to_pb(data_serial_type=data_serial_type,
+                                pickle_protocol=pickle_protocol).SerializeToString()
     if compress:
         ser_graph = zlib.compress(ser_graph)
     return ser_graph
@@ -294,7 +300,6 @@ def deserialize_graph(ser_graph, graph_cls=None):
     ser_graph_bin = to_binary(ser_graph)
     g = GraphDef()
     try:
-        ser_graph = ser_graph
         g.ParseFromString(ser_graph_bin)
         return graph_cls.from_pb(g)
     except DecodeError:
@@ -711,9 +716,9 @@ def check_chunks_unknown_shape(tileables, error_cls):
 def has_unknown_shape(tiled):
     if getattr(tiled, 'shape', None) is None:
         return False
-    if any(np.isnan(s) for s in tiled.shape):
+    if any(pd.isnull(s) for s in tiled.shape):
         return True
-    if any(np.isnan(s) for s in itertools.chain(*tiled.nsplits)):
+    if any(pd.isnull(s) for s in itertools.chain(*tiled.nsplits)):
         return True
     return False
 
@@ -1028,6 +1033,9 @@ class FixedSizeFileObject:
         self._cur = self._file_obj.tell()
         return result
 
+    def read1(self, size=None):
+        return self.read(size)
+
     def readline(self, size=None):
         result = self._file_obj.readline(self._get_size(size))
         self._cur = self._file_obj.tell()
@@ -1064,17 +1072,18 @@ class FixedSizeFileObject:
         return getattr(self._file_obj, item)
 
 
+def is_object_dtype(dtype):
+    try:
+        return np.issubdtype(dtype, np.object_) \
+               or np.issubdtype(dtype, np.unicode_) \
+               or np.issubdtype(dtype, np.bytes_)
+    except TypeError:  # pragma: no cover
+        return False
+
+
 def calc_object_overhead(chunk, shape):
     from .dataframe.core import DATAFRAME_CHUNK_TYPE, SERIES_CHUNK_TYPE, INDEX_CHUNK_TYPE
 
-    def is_object_dtype(dtype):
-        try:
-            return np.issubdtype(dtype, np.object_) \
-                   or np.issubdtype(dtype, np.unicode_) \
-                   or np.issubdtype(dtype, np.bytes_)
-        except TypeError:  # pragma: no cover
-            return False
-
     if not shape or np.isnan(shape[0]) or getattr(chunk, 'dtypes', None) is None:
         return 0
 
@@ -1152,3 +1161,104 @@ class Timer:
     def __exit__(self, *_):
         end = time.time()
         self.duration = end - self._start
+
+
+_io_quiet_local = threading.local()
+_io_quiet_lock = threading.Lock()
+
+
+class _QuietIOWrapper:
+    def __init__(self, wrapped):
+        self.wrapped = wrapped
+
+    def __getattr__(self, item):
+        return getattr(self.wrapped, item)
+
+    def write(self, d):
+        if getattr(_io_quiet_local, 'is_wrapped', False):
+            return 0
+        return self.wrapped.write(d)
+
+
+@contextmanager
+def quiet_stdio():
+    """Quiets standard outputs when inferring types of functions"""
+    with _io_quiet_lock:
+        _io_quiet_local.is_wrapped = True
+        sys.stdout = _QuietIOWrapper(sys.stdout)
+        sys.stderr = _QuietIOWrapper(sys.stderr)
+
+    try:
+        yield
+    finally:
+        with _io_quiet_lock:
+            sys.stdout = sys.stdout.wrapped
+            sys.stderr = sys.stderr.wrapped
+            if not isinstance(sys.stdout, _QuietIOWrapper):
+                _io_quiet_local.is_wrapped = False
+
+
+def implements(f):
+    def decorator(g):
+        g.__doc__ = f.__doc__
+        return g
+
+    return decorator
+
+
+def stringify_path(path: Union[str, os.PathLike]) -> str:
+    """
+    Convert *path* to a string or unicode path if possible.
+    """
+    if isinstance(path, str):
+        return path
+
+    # checking whether path implements the filesystem protocol
+    try:
+        return path.__fspath__()
+    except AttributeError:
+        raise TypeError("not a path-like object")
+
+
+def find_objects(nested, types):
+    found = []
+    stack = [nested]
+
+    while len(stack) > 0:
+        it = stack.pop()
+        if isinstance(it, types):
+            found.append(it)
+            continue
+
+        if isinstance(it, (list, tuple, set)):
+            stack.extend(list(it)[::-1])
+        elif isinstance(it, dict):
+            stack.extend(list(it.values())[::-1])
+
+    return found
+
+
+def replace_objects(nested, mapping):
+    if not mapping:
+        return nested
+
+    if isinstance(nested, dict):
+        vals = list(nested.values())
+    else:
+        vals = list(nested)
+
+    new_vals = []
+    for val in vals:
+        if isinstance(val, (dict, list, tuple, set)):
+            new_val = replace_objects(val, mapping)
+        else:
+            try:
+                new_val = mapping.get(val, val)
+            except TypeError:
+                new_val = val
+        new_vals.append(new_val)
+
+    if isinstance(nested, dict):
+        return type(nested)((k, v) for k, v in zip(nested.keys(), new_vals))
+    else:
+        return type(nested)(new_vals)
