@@ -149,7 +149,8 @@ class Session(object):
             resp = self._req_session.get(graph_url, params={'wait_timeout': timeout})
         except requests.ConnectionError as ex:
             err_msg = str(ex)
-            if 'ConnectionResetError' in err_msg or 'Connection refused' in err_msg:
+            if 'ConnectionResetError' in err_msg or 'Connection refused' in err_msg or \
+                    'Connection aborted' in err_msg:
                 return False
             raise
 
@@ -197,36 +198,37 @@ class Session(object):
         graph = build_tileable_graph(run_tileables, set(self._executed_tileables.keys()))
         targets = [t.key for t in run_tileables]
 
-        targets_join = ','.join(targets)
-        session_url = self._endpoint + '/api/session/' + self._session_id
-        graph_json = graph.to_json(data_serial_type=self._serial_type, pickle_protocol=self._pickle_protocol)
+        if len(graph) > 0:
+            targets_join = ','.join(targets)
+            session_url = self._endpoint + '/api/session/' + self._session_id
+            graph_json = graph.to_json(data_serial_type=self._serial_type, pickle_protocol=self._pickle_protocol)
 
-        resp_json = self._submit_graph(graph_json, targets_join, names=name or '', compose=compose)
-        graph_key = resp_json['graph_key']
-        graph_url = f'{session_url}/graph/{graph_key}'
+            resp_json = self._submit_graph(graph_json, targets_join, names=name or '', compose=compose)
+            graph_key = resp_json['graph_key']
+            graph_url = f'{session_url}/graph/{graph_key}'
 
-        exec_start_time = time.time()
-        time_elapsed = 0
-        check_interval = options.check_interval
-        while timeout <= 0 or time_elapsed < timeout:
-            timeout_val = min(check_interval, timeout - time_elapsed) if timeout > 0 else check_interval
-            try:
-                if self._check_response_finished(graph_url, timeout_val):
-                    break
-            except KeyboardInterrupt:
-                resp = self._req_session.delete(graph_url)
-                if resp.status_code >= 400:
-                    raise ExecutionNotStopped(
-                        f'Failed to stop graph execution. Code: {resp.status_code}, '
-                        f'Reason: {resp.reason}, Content:\n{resp.text}')
-            finally:
-                time_elapsed = time.time() - exec_start_time
+            exec_start_time = time.time()
+            time_elapsed = 0
+            check_interval = options.check_interval
+            while timeout <= 0 or time_elapsed < timeout:
+                timeout_val = min(check_interval, timeout - time_elapsed) if timeout > 0 else check_interval
+                try:
+                    if self._check_response_finished(graph_url, timeout_val):
+                        break
+                except KeyboardInterrupt:
+                    resp = self._req_session.delete(graph_url)
+                    if resp.status_code >= 400:
+                        raise ExecutionNotStopped(
+                            f'Failed to stop graph execution. Code: {resp.status_code}, '
+                            f'Reason: {resp.reason}, Content:\n{resp.text}')
+                finally:
+                    time_elapsed = time.time() - exec_start_time
 
-        if 0 < timeout < time.time() - exec_start_time:
-            raise TimeoutError
+            if 0 < timeout < time.time() - exec_start_time:
+                raise TimeoutError
 
-        for t in tileables:
-            self._set_tileable_graph_key(t, graph_key)
+            for t in tileables:
+                self._set_tileable_graph_key(t, graph_key)
 
         if not fetch:
             return
@@ -313,9 +315,9 @@ class Session(object):
             raise ValueError(f'Failed to get tileable key from server. Code: {resp.status_code}, '
                              f'Reason: {resp.reason}, Content:\n{resp.text}')
         tileable_key = self._handle_json_response(resp)['tileable_key']
-        nsplits = self._get_tileable_nsplits(tileable_key)
+        nsplits, extra_meta = self._get_tileable_meta(tileable_key)
         shape = tuple(sum(s) for s in nsplits)
-        return TileableInfos(tileable_key, shape)
+        return TileableInfos(tileable_key, shape, extra_meta)
 
     def create_mutable_tensor(self, name, shape, dtype, fill_value=None, chunk_size=None, *_, **__):
         from ..tensor.utils import create_mutable_tensor
@@ -396,6 +398,14 @@ class Session(object):
         new_nsplits = self._handle_json_response(resp)
         return new_nsplits
 
+    def _get_tileable_meta(self, tileable_key):
+        session_url = f'{self._endpoint}/api/session/{self._session_id}'
+        graph_key = self._get_tileable_graph_key(tileable_key)
+        url = f'{session_url}/graph/{graph_key}/data/{tileable_key}?type=meta'
+        resp = self._req_session.get(url)
+        meta = self._handle_json_response(resp)
+        return pickle.loads(base64.b64decode(meta))  # nosec
+
     def _update_tileable_shape(self, tileable):
         tileable_key = tileable.key
         new_nsplits = self._get_tileable_nsplits(tileable_key)
