@@ -354,8 +354,7 @@ class ExecutionActor(WorkerActor):
                     return
 
                 raise exc[1].with_traceback(exc[2])
-            except (BrokenPipeError, ConnectionRefusedError, TimeoutError,
-                    WorkerDead, promise.PromiseTimeout):
+            except (BrokenPipeError, ConnectionRefusedError, WorkerDead):
                 self._resource_ref.detach_dead_workers([remote_addr], _tell=True, _wait=False)
                 raise DependencyMissing((session_id, chunk_key))
 
@@ -378,7 +377,7 @@ class ExecutionActor(WorkerActor):
             ).then(_finish_fetch)
 
         return promise.finished() \
-            .then(lambda *_: remote_disp_ref.acquire_free_slot('sender', _promise=True, _timeout=timeout)) \
+            .then(lambda *_: remote_disp_ref.acquire_free_slot('sender', _promise=True)) \
             .then(_fetch_step) \
             .catch(_handle_network_error)
 
@@ -486,7 +485,7 @@ class ExecutionActor(WorkerActor):
 
         calc_device = calc_device or 'cpu'
         if calc_device == 'cpu':  # pragma: no cover
-            if options.vineyard.socket:
+            if options.vineyard.enabled:
                 preferred_data_device = DataStorageDevice.VINEYARD
             else:
                 preferred_data_device = DataStorageDevice.SHARED_MEMORY
